# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "numpy==1.23.2",
#     "pandas==2.2.0",
# ]
# ///

"""Create the BoolQ-pt dataset and upload to HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi

RANDOM_STATE = 4242
TRAIN_SIZE, VAL_SIZE, TEST_SIZE = 1024, 256, 2048
ORIGINAL_REPO_ID = "PORTULAN/extraglue"
FINAL_REPO_ID = "EuroEval/boolq-pt"


def main() -> None:
    """Create the dataset and upload to HF Hub."""
    ds_raw = load_dataset(ORIGINAL_REPO_ID, name="boolq_pt-PT")
    assert isinstance(ds_raw, DatasetDict)

    # Combine all splits to avoid duplicates across splits
    all_data = []
    for split in ["train", "test", "validation"]:
        df = ds_raw[split].to_pandas()
        assert isinstance(df, pd.DataFrame)
        all_data.append(df)

    combined_df = pd.concat(all_data, ignore_index=True)

    # Remove duplicates based on passage + question combination
    combined_df = combined_df.drop_duplicates(
        subset=["passage", "question"], keep="first"
    )

    # Shuffle the data
    combined_df = combined_df.sample(frac=1, random_state=RANDOM_STATE).reset_index(
        drop=True
    )

    # Calculate total needed samples
    total_needed = TRAIN_SIZE + VAL_SIZE + TEST_SIZE
    if len(combined_df) < total_needed:
        raise ValueError(
            f"Not enough unique samples. Available: {len(combined_df)}, "
            f"needed: {total_needed}"
        )

    # Split the data
    train_df = combined_df.iloc[:TRAIN_SIZE]
    val_df = combined_df.iloc[TRAIN_SIZE : TRAIN_SIZE + VAL_SIZE]
    test_df = combined_df.iloc[
        TRAIN_SIZE + VAL_SIZE : TRAIN_SIZE + VAL_SIZE + TEST_SIZE
    ]

    # Transform datasets
    train_df = transform_dataset(train_df)
    test_df = transform_dataset(test_df)
    val_df = transform_dataset(val_df)

    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    assert not set(dataset["train"]["text"]) & set(dataset["val"]["text"])
    assert not set(dataset["train"]["text"]) & set(dataset["test"]["text"])
    assert not set(dataset["val"]["text"]) & set(dataset["test"]["text"])

    HfApi().delete_repo(FINAL_REPO_ID, repo_type="dataset", missing_ok=True)

    dataset.push_to_hub(FINAL_REPO_ID, private=True)


def transform_dataset(df: pd.DataFrame) -> pd.DataFrame:
    """Transform dataset to multiple choice format.

    Args:
        df: DataFrame with columns 'passage', 'question', and 'label'.

    Returns:
        DataFrame with columns 'text' and 'label'.
    """
    texts = []
    labels = []

    for _, row in df.iterrows():
        text = (
            f"Texto: {row['passage']}\n"
            f"Pergunta: {row['question']}\n"
            f"Opções:\n"
            f"a. sim\n"
            f"b. não"
        )

        # Label 1 (yes) -> "a", Label 0 (no) -> "b"
        correct_label = "a" if row["label"] == 1 else "b"

        texts.append(text)
        labels.append(correct_label)

    return pd.DataFrame({"text": texts, "label": labels})


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Hungarian summarisation dataset based on hun-sum-chatml-5k."""

import json
import os

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from dotenv import load_dotenv
from huggingface_hub import HfApi
from openai import OpenAI
from openai.types.chat import ChatCompletionUserMessageParam
from pydantic import BaseModel

from .constants import MAX_NUM_CHARS_IN_ARTICLE, MIN_NUM_CHARS_IN_ARTICLE

load_dotenv()

CACHE_FILE = "summary_cache.json"


class SummaryValidation(BaseModel):
    """Structured output for the summary validation.

    Args:
        is_valid_summary: True if the summary aligns with the text, False otherwise.
    """

    is_valid_summary: bool


def load_cache() -> dict:
    """Load cache from CACHE_FILE if it exists."""
    try:
        with open(CACHE_FILE, "r") as cache_file:
            return json.load(cache_file)
    except FileNotFoundError:
        return {}


def save_cache(cache: dict) -> None:
    """Save cache to CACHE_FILE."""
    with open(CACHE_FILE, "w") as cache_file:
        json.dump(cache, cache_file, indent=4)


summary_cache = load_cache()

client = OpenAI(api_key=os.environ["OPENAI_API_KEY"])


def main() -> None:
    """Create the Hungarian summarisation mini dataset and upload to HF Hub."""
    dataset_id = "ariel-ml/hun-sum-chatml-5k"

    dataset = load_dataset(dataset_id, token=True)
    assert isinstance(dataset, DatasetDict)

    # The dataset has splits: train, val, test
    # Each sample has: title, lead, article
    # We want: text = "{title}\n\n{article}", target_text = lead

    def make_columns(sample: dict) -> dict:
        """Map the dataset to have the text and target_text columns.

        Args:
            sample: A sample from the dataset.

        Returns:
            A sample with the text and target_text columns.
        """
        sample["text"] = f"{sample['title']}\n\n{sample['article']}"
        sample["target_text"] = sample["lead"]
        return sample

    dataset = dataset.map(make_columns)

    train_df = dataset["train"].to_pandas()
    val_df = dataset["validation"].to_pandas()
    test_df = dataset["test"].to_pandas()

    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    train_df = process(df=train_df)
    val_df = process(df=val_df)
    test_df = process(df=test_df)

    train_df_final, val_df_final, test_df_final = create_splits(
        train_df=train_df, val_df=val_df, test_df=test_df
    )

    # Collect datasets in a dataset dictionary
    mini_dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df_final, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df_final, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df_final, split=Split.TEST),
        }
    )

    # Create dataset ID
    mini_dataset_id = "EuroEval/hun-sum-mini"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(mini_dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    mini_dataset.push_to_hub(mini_dataset_id, private=True)


def process(df: pd.DataFrame) -> pd.DataFrame:
    """Process the dataframe.

    Args:
        df: The dataframe to process.

    Returns:
        The processed dataframe.
    """
    # Validate samples using an LLM
    df["is_valid_summary"] = df.apply(_text_summary_alignment, axis=1)
    df = df.loc[df["is_valid_summary"]]

    keep_columns = ["text", "target_text"]
    df = df[keep_columns]

    # Only work with samples where the text is not very large or small
    lengths = df.text.str.len()
    lower_bound = MIN_NUM_CHARS_IN_ARTICLE
    upper_bound = MAX_NUM_CHARS_IN_ARTICLE
    df = df[lengths.between(lower_bound, upper_bound)]
    df = df.reset_index(drop=True)
    return df


def _text_summary_alignment(row: pd.Series) -> bool:
    """Check if the summary aligns with the text using an LLM, with caching.

    Args:
        row: A row from the dataframe.

    Returns:
        True if the summary aligns with the text, False otherwise.
    """
    text = row["text"]
    summary = row["target_text"]
    if summary in summary_cache:
        return summary_cache[summary]

    messages: list[ChatCompletionUserMessageParam] = list()
    user_message = ChatCompletionUserMessageParam(
        role="user",
        content=(
            f"Does the summary <summary>{summary}</summary> align with the text "
            f"<text>{text}</text> and represent a true summary?"
        ),
    )
    messages.append(user_message)
    completion = client.beta.chat.completions.parse(
        model="gpt-4o", messages=messages, response_format=SummaryValidation
    )
    parsed = completion.choices[0].message.parsed
    if parsed is None:
        raise ValueError("Parsed response is None")
    is_valid_summary = parsed.is_valid_summary

    # Cache the result
    summary_cache[summary] = is_valid_summary
    save_cache(cache=summary_cache)

    return is_valid_summary


def create_splits(
    train_df: pd.DataFrame, val_df: pd.DataFrame, test_df: pd.DataFrame
) -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Create splits.

    Args:
        train_df: The training dataframe.
        val_df: The validation dataframe.
        test_df: The test dataframe.

    Returns:
        The final training, validation, and test dataframes.
    """
    # Split sizes
    train_size = 1024
    val_size = 256
    test_size = 2048

    # Create train split
    train_df_final = train_df.sample(n=train_size, random_state=4242)
    train_df_remaining = train_df[~train_df.index.isin(train_df_final.index)]

    # Create validation split
    n_missing_val_samples = val_size - len(val_df)
    val_df_additional = train_df_remaining.sample(
        n=n_missing_val_samples, random_state=4242
    )
    val_df_final = pd.concat([val_df, val_df_additional], ignore_index=True)
    train_df_remaining = train_df_remaining.loc[
        ~train_df_remaining.index.isin(val_df_additional.index)
    ]

    # Create test split
    n_missing_test_samples = test_size - len(test_df)
    test_df_additional = train_df_remaining.sample(
        n=n_missing_test_samples, random_state=4242
    )
    test_df_final = pd.concat([test_df, test_df_additional], ignore_index=True)

    # Reset indices
    train_df_final = train_df_final.reset_index(drop=True)
    val_df_final = val_df_final.reset_index(drop=True)
    test_df_final = test_df_final.reset_index(drop=True)

    assert isinstance(train_df_final, pd.DataFrame)
    assert isinstance(val_df_final, pd.DataFrame)
    assert isinstance(test_df_final, pd.DataFrame)

    return train_df_final, val_df_final, test_df_final


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the PSC (Polish Summaries Corpus) summarization dataset."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi

from .constants import MAX_NUM_CHARS_IN_ARTICLE, MIN_NUM_CHARS_IN_ARTICLE


def main() -> None:
    """Create the PSC summarization dataset and upload to HF Hub."""
    dataset_id = "community-datasets/psc"

    dataset = load_dataset(dataset_id, token=True)
    assert isinstance(dataset, DatasetDict)

    # Rename columns to match expected format
    dataset = dataset.rename_columns(
        column_mapping={"extract_text": "text", "summary_text": "target_text"}
    )

    # Work primarily with the train split since test split has no proper labels
    train_df = dataset["train"].to_pandas()
    assert isinstance(train_df, pd.DataFrame)

    # Remove the label column since we don't need it for summarization
    if "label" in train_df.columns:
        train_df = train_df.drop(columns=["label"])

    # Only work with samples where the text is not very large or small
    text_lengths = train_df.text.str.len()
    lower_bound = MIN_NUM_CHARS_IN_ARTICLE
    upper_bound = MAX_NUM_CHARS_IN_ARTICLE
    train_df = train_df[text_lengths.between(lower_bound, upper_bound)]

    # Create validation split from train data
    val_size = 256
    val_df = train_df.sample(n=val_size, random_state=4242)
    val_df = val_df.reset_index(drop=True)

    # Remove validation samples from train data and reset index
    remaining_train = train_df.drop(val_df.index.tolist()).reset_index(drop=True)

    # Create test split from remaining train data
    test_size = 2048
    test_df = remaining_train.sample(
        n=min(test_size, len(remaining_train)), random_state=4242
    )
    test_df = test_df.reset_index(drop=True)

    # Remove test samples from train data
    remaining_train = remaining_train.drop(test_df.index.tolist()).reset_index(
        drop=True
    )

    # Create final train split
    train_size = 1024
    train_df = remaining_train.sample(
        n=min(train_size, len(remaining_train)), random_state=4242
    )
    train_df = train_df.reset_index(drop=True)

    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    dataset_id = "EuroEval/psc-mini"

    # Push the dataset to the Hugging Face Hub
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



"""All Bulgarian dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import BULGARIAN
from ..tasks import COMMON_SENSE, KNOW, LA, NER, RC, SENT

### Official datasets ###

CINEXIO_CONFIG = DatasetConfig(
    name="cinexio",
    pretty_name="Cinexio",
    source="EuroEval/cinexio-mini",
    task=SENT,
    languages=[BULGARIAN],
)

SCALA_BG_CONFIG = DatasetConfig(
    name="scala-bg",
    pretty_name="ScaLA-bg",
    source="EuroEval/scala-bg",
    task=LA,
    languages=[BULGARIAN],
)

BG_NER_BSNLP_CONFIG = DatasetConfig(
    name="bg-ner-bsnlp",
    pretty_name="BG-NER-BSNLp",
    source="EuroEval/bg-ner-bsnlp-mini",
    task=NER,
    languages=[BULGARIAN],
)

MULTI_WIKI_QA_BG_CONFIG = DatasetConfig(
    name="multi-wiki-qa-bg",
    pretty_name="MultiWikiQA-bg",
    source="EuroEval/multi-wiki-qa-bg-mini",
    task=RC,
    languages=[BULGARIAN],
)

EXAMS_BG_CONFIG = DatasetConfig(
    name="exams-bg",
    pretty_name="Exams-bg",
    source="EuroEval/exams-bg-mini",
    task=KNOW,
    languages=[BULGARIAN],
)

WINOGRANDE_BG_CONFIG = DatasetConfig(
    name="winogrande-bg",
    pretty_name="Winogrande-bg",
    source="EuroEval/winogrande-bg",
    task=COMMON_SENSE,
    languages=[BULGARIAN],
    _labels=["a", "b"],
)



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "polars==1.31.0",
#     "requests==2.32.3",
#     "scikit-learn<1.6.0",
# ]
# ///

"""Create the MMLU-mini datasets and upload them to the HF Hub."""

from collections import Counter

import pandas as pd
import polars as pl  # type: ignore[missing-import]
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi
from sklearn.model_selection import train_test_split

from .constants import (
    CHOICES_MAPPING,
    MAX_NUM_CHARS_IN_INSTRUCTION,
    MAX_NUM_CHARS_IN_OPTION,
    MAX_REPETITIONS,
    MIN_NUM_CHARS_IN_INSTRUCTION,
    MIN_NUM_CHARS_IN_OPTION,
)

LANGUAGES = [
    "da",
    "de",
    "ca",
    "en",
    "es",
    "fr",
    "hu",
    "is",
    "it",
    "nl",
    "no",
    "pt",
    "sk",
    "sl",
    "sr",
    "sv",
]


MMLUX_SUBSET_IDS = {"pt": "PT-PT", "sl": "SL"}


def main() -> None:
    """Create the MMLU-mini datasets and upload them to the HF Hub."""
    # Define the base download URL
    repo_id = "alexandrainst/m_mmlu"

    for language in LANGUAGES:
        # Download the dataset
        try:
            dataset = load_dataset(path=repo_id, name=language, token=True)
        except ValueError as e:
            if language == "no":
                dataset = load_dataset(path=repo_id, name="nb", token=True)
            elif language in MMLUX_SUBSET_IDS:
                subset_id = MMLUX_SUBSET_IDS[language]
                dataset = load_mmlux_dataset(subset_id=subset_id)
            else:
                raise e
        assert isinstance(dataset, DatasetDict)

        # Convert the dataset to a dataframe
        train_df = dataset["train"].to_pandas()
        val_df = dataset["val"].to_pandas()
        test_df = dataset["test"].to_pandas()
        assert isinstance(train_df, pd.DataFrame)
        assert isinstance(val_df, pd.DataFrame)
        assert isinstance(test_df, pd.DataFrame)

        # Concatenate the splits
        df = pd.concat([train_df, val_df, test_df], ignore_index=True)

        # Rename the columns
        df.rename(columns=dict(answer="label"), inplace=True)

        # Remove the samples with overly short or long texts
        df = df[
            (df.instruction.str.len() >= MIN_NUM_CHARS_IN_INSTRUCTION)
            & (df.instruction.str.len() <= MAX_NUM_CHARS_IN_INSTRUCTION)
            & (df.option_a.str.len() >= MIN_NUM_CHARS_IN_OPTION)
            & (df.option_a.str.len() <= MAX_NUM_CHARS_IN_OPTION)
            & (df.option_b.str.len() >= MIN_NUM_CHARS_IN_OPTION)
            & (df.option_b.str.len() <= MAX_NUM_CHARS_IN_OPTION)
            & (df.option_c.str.len() >= MIN_NUM_CHARS_IN_OPTION)
            & (df.option_c.str.len() <= MAX_NUM_CHARS_IN_OPTION)
            & (df.option_d.str.len() >= MIN_NUM_CHARS_IN_OPTION)
            & (df.option_d.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        ]

        def is_repetitive(text: str) -> bool:
            """Return True if the text is repetitive."""
            max_repetitions = max(Counter(text.split()).values())
            return max_repetitions > MAX_REPETITIONS

        # Remove overly repetitive samples
        df = df[
            ~df.instruction.apply(is_repetitive)
            & ~df.option_a.apply(is_repetitive)
            & ~df.option_b.apply(is_repetitive)
            & ~df.option_c.apply(is_repetitive)
            & ~df.option_d.apply(is_repetitive)
        ]
        assert isinstance(df, pd.DataFrame)

        # Extract the category as a column
        df["category"] = df["id"].str.split("/").str[0]

        # Make a `text` column with all the options in it
        df["text"] = [
            row.instruction.replace("\n", " ").strip() + "\n"
            f"{CHOICES_MAPPING[language]}:\n"
            "a. " + row.option_a.replace("\n", " ").strip() + "\n"
            "b. " + row.option_b.replace("\n", " ").strip() + "\n"
            "c. " + row.option_c.replace("\n", " ").strip() + "\n"
            "d. " + row.option_d.replace("\n", " ").strip()
            for _, row in df.iterrows()
        ]

        # Make the `label` column case-consistent with the `text` column
        df.label = df.label.str.lower()

        # Only keep the `text`, `label` and `category` columns
        df = df[["text", "label", "category"]]

        # Remove duplicates
        df.drop_duplicates(inplace=True)
        df.reset_index(drop=True, inplace=True)

        # Create validation split
        val_size = 256
        traintest_arr, val_arr = train_test_split(
            df, test_size=val_size, random_state=4242, stratify=df.category
        )
        traintest_df = pd.DataFrame(traintest_arr, columns=df.columns)
        val_df = pd.DataFrame(val_arr, columns=df.columns)

        # Create test split
        test_size = 2048
        train_arr, test_arr = train_test_split(
            traintest_df,
            test_size=test_size,
            random_state=4242,
            stratify=traintest_df.category,
        )
        train_df = pd.DataFrame(train_arr, columns=df.columns)
        test_df = pd.DataFrame(test_arr, columns=df.columns)

        # Create train split
        train_size = 1024
        train_df = train_df.sample(train_size, random_state=4242)

        # Reset the index
        train_df = train_df.reset_index(drop=True)
        val_df = val_df.reset_index(drop=True)
        test_df = test_df.reset_index(drop=True)

        # Collect datasets in a dataset dictionary
        dataset = DatasetDict(
            {
                "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
                "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
                "test": Dataset.from_pandas(test_df, split=Split.TEST),
            }
        )

        # Create dataset ID
        if language == "en":
            dataset_id = "EuroEval/mmlu-mini"
        else:
            dataset_id = f"EuroEval/mmlu-{language}-mini"

        # Remove the dataset from Hugging Face Hub if it already exists
        HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

        # Push the dataset to the Hugging Face Hub
        dataset.push_to_hub(dataset_id, private=True)


def load_mmlux_dataset(subset_id: str = "PT-PT") -> DatasetDict:
    """Load and process a subset split from LumiOpen/opengpt-x_mmlux.

    Returns:
        DatasetDict: Hugging Face DatasetDict with train, val, and test splits.
    """

    def _process_split(split: str) -> pl.DataFrame:
        """Process a single split of the Portuguese MMLU dataset.

        Args:
            split (str): The split name ("dev", "validation", or "test").

        Returns:
            polars.DataFrame: Processed DataFrame for the split.
        """
        return (
            pl.read_ndjson(
                f"hf://datasets/LumiOpen/opengpt-x_mmlux/*{subset_id}*{split}.jsonl"
            )
            .with_columns(
                pl.col("id").str.split("/").list.get(0).alias("category"),
                (pl.col("id") + f"/{split}").alias("id"),
            )
            .rename({"question": "instruction"})
            .with_columns(
                [
                    pl.col("choices").list.get(0).alias("option_a"),
                    pl.col("choices").list.get(1).alias("option_b"),
                    pl.col("choices").list.get(2).alias("option_c"),
                    pl.col("choices").list.get(3).alias("option_d"),
                ]
            )
            .with_columns(
                pl.col("answer").map_elements(
                    lambda x: {0: "a", 1: "b", 2: "c", 3: "d"}[x],
                    return_dtype=pl.String,
                )
            )
            .drop("choices")
            .drop("category")
        )

    train_df = _process_split("dev").to_pandas()
    val_df = _process_split("validation").to_pandas()
    test_df = _process_split("test").to_pandas()

    return DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )


if __name__ == "__main__":
    main()



"""Tests for the `types` module."""

import pytest

from euroeval.types import is_list_of_int, is_list_of_list_of_int, is_list_of_str


@pytest.mark.parametrize(
    argnames=["obj", "expected"],
    argvalues=[
        ([1, 2, 3], True),
        ([1.0, 2.0, 3.0], False),
        (["a", "b", "c"], False),
        (["a", 1, 2.0], False),
        ([], True),
    ],
)
def test_is_list_of_int(obj: list, expected: bool) -> None:
    """Test the `is_list_of_int` function."""
    assert is_list_of_int(x=obj) == expected


@pytest.mark.parametrize(
    argnames=["obj", "expected"],
    argvalues=[
        ([1, 2, 3], False),
        ([[1, 2, 3], [4, 5, 6]], True),
        ([[1.0, 2.0, 3.0], [4.0, 5.0, 6.0]], False),
        ([["a", "b", "c"], ["d", "e", "f"]], False),
        ([["a", 1, 2.0], [3, 4, 5]], False),
        ([[], []], True),
    ],
)
def test_is_list_of_list_of_int(obj: list, expected: bool) -> None:
    """Test the `is_list_of_int` function."""
    assert is_list_of_list_of_int(x=obj) == expected


@pytest.mark.parametrize(
    argnames=["obj", "expected"],
    argvalues=[
        (["a", "b", "c"], True),
        (["a", 1, 2.0], False),
        ([1, 2, 3], False),
        ([], True),
    ],
)
def test_is_list_of_str(obj: list, expected: bool) -> None:
    """Test the `is_list_of_str` function."""
    assert is_list_of_str(x=obj) == expected



"""Tests for scripts/create_scala.py."""

import re
from pathlib import Path

import pytest
from pytest import FixtureRequest

from scripts.create_scala import prepare_df
from scripts.load_ud_pos import load_ud_pos


@pytest.fixture(scope="module")
def scala_test_data_path(request: FixtureRequest) -> Path:
    """Path to the test data directory, relative to this test file.

    Args:
        request: The pytest fixture request.

    Returns:
        The path to the test data directory.
    """
    return Path(request.path).parent / "test_data"


@pytest.mark.parametrize(
    argnames=["filename", "wrong_words", "correct_word"],
    argvalues=(
        (
            "pl_pdb-ud-train.conllu.aux_clitic_01",
            ["postanowili", "śmy"],
            "postanowiliśmy",
        ),
        ("pl_pdb-ud-train.conllu.aux_clitic_02", ["nadział", "em"], "nadziałem"),
        ("pl_pdb-ud-train.conllu.aux_clitic_02", ["zarzucił", "em"], "zarzuciłem"),
        ("pl_pdb-ud-train.conllu.aux_clitic_03", ["chciał", "by", "m"], "chciałbym"),
        ("pl_pdb-ud-train.conllu.aux_clitic_03", ["aby", "ś"], "abyś"),
        ("de_gsd-ud-train.conllu.adp_det", ["in", "dem"], "im"),
        (
            "en_gum-ud-train.conllu.case",
            [re.compile(r"(^|\s)'($|\s)"), re.compile(r"\bGalois(\s|$)")],
            re.compile(r"\bGalois'"),
        ),
    ),
)
def test_load_ud_pos(
    filename: str,
    wrong_words: list[str | re.Pattern],
    correct_word: str | re.Pattern,
    scala_test_data_path: Path,
) -> None:
    """Test of the universal dependencies can be loaded correctly."""
    wrong_patterns = [
        wrong_word
        if isinstance(wrong_word, re.Pattern)
        else re.compile(rf"\b{wrong_word}\b", flags=re.IGNORECASE)
        for wrong_word in wrong_words
    ]
    correct_pattern = (
        correct_word
        if isinstance(correct_word, re.Pattern)
        else re.compile(rf"\b{correct_word}\b", flags=re.IGNORECASE)
    )

    dfs = load_ud_pos(
        str(scala_test_data_path / filename),
        str(scala_test_data_path / "empty.file"),
        str(scala_test_data_path / "empty.file"),
    )

    ds = prepare_df(df=dfs["train"], split="train")
    for row in ds:
        assert isinstance(row, dict), (
            f"Rows need to be dictionaries, but found {type(row)} for the row {row}."
        )
        if row["label"] == "incorrect":
            for wrong_pattern in wrong_patterns:
                assert wrong_pattern.search(row["text"]) is None, (
                    f"The wrong pattern {wrong_pattern!r} appeared in the text "
                    f"{row['text']!r}, which was not allowed."
                )
        else:
            assert correct_pattern.search(row["text"]) is not None, (
                f"The correct pattern {correct_pattern!r} did not appear in the text "
                f"{row['text']!r}, which is required."
            )



"""Functions related to the loading of the data."""

import collections.abc as c
import logging
import sys
import time
import typing as t

import requests
from datasets import DatasetDict, load_dataset
from datasets.exceptions import DatasetsError
from huggingface_hub.errors import HfHubHTTPError
from numpy.random import Generator

from .constants import SUPPORTED_FILE_FORMATS_FOR_LOCAL_DATASETS
from .exceptions import HuggingFaceHubDown, InvalidBenchmark
from .logging_utils import log, no_terminal_output
from .tasks import EUROPEAN_VALUES
from .utils import unscramble

if t.TYPE_CHECKING:
    from datasets import Dataset

    from .data_models import BenchmarkConfig, DatasetConfig


def load_data(
    rng: Generator, dataset_config: "DatasetConfig", benchmark_config: "BenchmarkConfig"
) -> list["DatasetDict"]:
    """Load the raw bootstrapped datasets.

    Args:
        rng:
            The random number generator to use.
        dataset_config:
            The configuration for the dataset.
        benchmark_config:
            The configuration for the benchmark.

    Returns:
        A list of bootstrapped datasets, one for each iteration.

    Raises:
        InvalidBenchmark:
            If the dataset cannot be loaded.
        HuggingFaceHubDown:
            If the Hugging Face Hub is down.
    """
    dataset = load_raw_data(
        dataset_config=dataset_config, cache_dir=benchmark_config.cache_dir
    )

    if not benchmark_config.evaluate_test_split and "val" in dataset:
        dataset["test"] = dataset["val"]

    # Remove empty examples from the datasets
    for text_feature in ["tokens", "text"]:
        for split in dataset_config.splits:
            if text_feature in dataset[split].features:
                dataset = dataset.filter(lambda x: len(x[text_feature]) > 0)

    # If we are testing then truncate the test set, unless we need the full set for
    # evaluation
    if hasattr(sys, "_called_from_test") and dataset_config.task != EUROPEAN_VALUES:
        dataset["test"] = dataset["test"].select(range(1))  # type: ignore[unsupported-operation]

    # Bootstrap the splits, if applicable
    if dataset_config.bootstrap_samples:
        bootstrapped_splits: dict[str, c.Sequence["Dataset"]] = dict()
        for split in dataset_config.splits:
            bootstrap_indices = rng.integers(
                0,
                len(dataset[split]),
                size=(benchmark_config.num_iterations, len(dataset[split])),
            )
            bootstrapped_splits[split] = [  # type: ignore[unsupported-operation]
                dataset[split].select(bootstrap_indices[idx])
                for idx in range(benchmark_config.num_iterations)
            ]
        datasets = [
            DatasetDict(  # type: ignore[no-matching-overload]
                {
                    split: bootstrapped_splits[split][idx]
                    for split in dataset_config.splits
                }
            )
            for idx in range(benchmark_config.num_iterations)
        ]
    else:
        datasets = [dataset] * benchmark_config.num_iterations

    return datasets


def load_raw_data(dataset_config: "DatasetConfig", cache_dir: str) -> "DatasetDict":
    """Load the raw dataset.

    Args:
        dataset_config:
            The configuration for the dataset.
        cache_dir:
            The directory to cache the dataset.

    Returns:
        The dataset.
    """
    # Case where the dataset source is a Hugging Face ID
    if isinstance(dataset_config.source, str):
        num_attempts = 5
        for _ in range(num_attempts):
            try:
                with no_terminal_output():
                    dataset = load_dataset(
                        path=dataset_config.source.split("::")[0],
                        name=(
                            dataset_config.source.split("::")[1]
                            if "::" in dataset_config.source
                            else None
                        ),
                        cache_dir=cache_dir,
                        token=unscramble("XbjeOLhwebEaSaDUMqqaPaPIhgOcyOfDpGnX_"),
                    )
                break
            except (
                FileNotFoundError,
                ConnectionError,
                DatasetsError,
                requests.ConnectionError,
                requests.ReadTimeout,
            ) as e:
                log(
                    f"Failed to load dataset {dataset_config.source!r}, due to "
                    f"the following error: {e}. Retrying...",
                    level=logging.DEBUG,
                )
                time.sleep(1)
                continue
            except HfHubHTTPError:
                raise HuggingFaceHubDown()
        else:
            raise InvalidBenchmark(
                f"Failed to load dataset {dataset_config.source!r} after "
                f"{num_attempts} attempts. Run with verbose mode to see the individual "
                "errors."
            )

    # Case where the dataset source is a dictionary with keys "train", "val" and "test",
    # with the values pointing to local CSV files
    else:
        data_files = {
            split: dataset_config.source[split]
            for split in dataset_config.splits
            if split in dataset_config.source
        }

        # Get the file extension and ensure that all files have the same extension
        file_extensions = {
            split: dataset_config.source[split].split(".")[-1]
            for split in dataset_config.splits
            if split in dataset_config.source
        }
        if len(set(file_extensions.values())) != 1:
            raise InvalidBenchmark(
                "All data files in a custom dataset must have the same file extension. "
                f"Got the extensions {', '.join(file_extensions.values())} for the "
                f"dataset {dataset_config.name!r}."
            )
        file_extension = list(file_extensions.values())[0]

        # Check that the file extension is supported
        if file_extension not in SUPPORTED_FILE_FORMATS_FOR_LOCAL_DATASETS:
            raise InvalidBenchmark(
                "Unsupported file extension for custom dataset. Supported file "
                "extensions are "
                f"{', '.join(SUPPORTED_FILE_FORMATS_FOR_LOCAL_DATASETS)}, but got "
                f"{file_extension!r}."
            )

        # Load the dataset
        with no_terminal_output():
            dataset = load_dataset(
                path=file_extension, data_files=data_files, cache_dir=cache_dir
            )

    assert isinstance(dataset, DatasetDict)  # type: ignore[used-before-def]
    missing_keys = [key for key in dataset_config.splits if key not in dataset]
    if missing_keys:
        raise InvalidBenchmark(
            "The dataset is missing the following required splits: "
            f"{', '.join(missing_keys)}"
        )
    return DatasetDict({key: dataset[key] for key in dataset_config.splits})  # type: ignore[no-matching-overload]



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
#     "scikit-learn<1.6.0",
# ]
# ///

"""Create the Life in the UK multiple choice dataset and upload it to the HF Hub."""

from collections import Counter

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi
from sklearn.model_selection import train_test_split

from .constants import (
    CHOICES_MAPPING,
    MAX_NUM_CHARS_IN_INSTRUCTION,
    MAX_NUM_CHARS_IN_OPTION,
    MAX_REPETITIONS,
    MIN_NUM_CHARS_IN_INSTRUCTION,
    MIN_NUM_CHARS_IN_OPTION,
)


def main() -> None:
    """Create the Life in the UK multiple choice dataset and upload it to the HF Hub."""
    # Define the dataset ID
    repo_id = "oliverkinch/life-in-the-uk-multiple-choice"

    # Download the dataset
    dataset = load_dataset(path=repo_id)
    assert isinstance(dataset, DatasetDict)

    # Convert the dataset to a dataframe
    df = dataset["train"].to_pandas()
    assert isinstance(df, pd.DataFrame)

    # Rename the columns to match EuroEval format
    df.rename(columns=dict(answer="label"), inplace=True)

    # Remove all samples with a null value in any of the option columns
    df = df[
        df["a"].notnull() & df["b"].notnull() & df["c"].notnull() & df["d"].notnull()
    ]

    # Remove the samples with overly short or long texts
    df = df.loc[
        (df.question.str.len() >= MIN_NUM_CHARS_IN_INSTRUCTION)
        & (df.question.str.len() <= MAX_NUM_CHARS_IN_INSTRUCTION)
        & (df.a.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.a.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.b.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.b.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.c.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.c.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.d.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.d.str.len() <= MAX_NUM_CHARS_IN_OPTION)
    ]

    def is_repetitive(text: str) -> bool:
        """Return True if the text is repetitive."""
        max_repetitions = max(Counter(text.split()).values())
        return max_repetitions > MAX_REPETITIONS

    # Remove overly repetitive samples
    df = df.loc[
        ~df.question.apply(is_repetitive)
        & ~df.a.apply(is_repetitive)
        & ~df.b.apply(is_repetitive)
        & ~df.c.apply(is_repetitive)
        & ~df.d.apply(is_repetitive)
    ]

    # Make a `text` column with all the options in it
    df["text"] = [
        row.question.replace("\n", " ").strip() + "\n"
        f"{CHOICES_MAPPING['en']}:\n"
        "a. " + row.a.replace("\n", " ").strip() + "\n"
        "b. " + row.b.replace("\n", " ").strip() + "\n"
        "c. " + row.c.replace("\n", " ").strip() + "\n"
        "d. " + row.d.replace("\n", " ").strip()
        for _, row in df.iterrows()
    ]

    # Make the `label` column case-consistent with the `text` column
    df.label = df.label.str.lower()

    # Only keep the `text` and `label` columns
    df = df[["text", "label"]]

    # Remove duplicates
    df.drop_duplicates(inplace=True)
    df.reset_index(drop=True, inplace=True)

    # Create validation split
    val_size = 256
    traintest_arr, val_arr = train_test_split(df, test_size=val_size, random_state=4242)
    traintest_df = pd.DataFrame(traintest_arr, columns=df.columns)
    val_df = pd.DataFrame(val_arr, columns=df.columns)

    # Create test split
    test_size = 512
    train_arr, test_arr = train_test_split(
        traintest_df, test_size=test_size, random_state=4242
    )
    train_df = pd.DataFrame(train_arr, columns=df.columns)
    test_df = pd.DataFrame(test_arr, columns=df.columns)

    # Create train split
    train_size = 438
    train_df = train_df.sample(train_size, random_state=4242)

    # Reset the index
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/life-in-the-uk"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the HAREM NER dataset and upload it to the HF Hub."""

import logging
import re
import urllib.request
from collections import Counter

import pandas as pd
from datasets import Dataset, DatasetDict, Split
from huggingface_hub import HfApi

logging.basicConfig(format="%(asctime)s ⋅ %(message)s", level=logging.INFO)
logger = logging.getLogger("create_harem")

# Constants for dataset sizes
TRAIN_SIZE = 1024
VAL_SIZE = 256
TEST_SIZE = 1024
RANDOM_STATE = 4242

# URL for HAREM dataset
URL = (
    "https://raw.githubusercontent.com/davidsbatista/NER-datasets/master/Portuguese/"
    "HAREM/ColeccaoDouradaHAREM.txt"
)

# Regular expressions for parsing
TAG_RE = re.compile(r"<(/?)(\w+)(?:\s[^>]*)?>")
TOKEN_RE = re.compile(r"\w+|[^\w\s]", re.UNICODE)
SENTENCE_RE = re.compile(r"[.!?]+")

# Tag mapping from HAREM to standard NER labels
TAG2LABEL: dict[str, str] = {
    "ORGANIZACAO": "ORG",
    "PESSOA": "PER",
    "LOCAL": "LOC",
    "TEMPO": "MISC",
    "ACONTECIMENTO": "MISC",
    "ABSTRACCAO": "MISC",
    "VALOR": "MISC",
    "VARIADO": "MISC",
    "OBRA": "MISC",
    "OMITIDO": "MISC",
    "COISA": "MISC",
    "ALT": "MISC",
}

# Label to ID mapping
LABEL2ID = {
    "O": 0,
    "B-PER": 1,
    "I-PER": 2,
    "B-ORG": 3,
    "I-ORG": 4,
    "B-LOC": 5,
    "I-LOC": 6,
    "B-MISC": 7,
    "I-MISC": 8,
}

# ID to label mapping for conversion
ID2LABEL = {v: k for k, v in LABEL2ID.items()}

# Counters for tracking
SEEN_TAGS: Counter = Counter()
SEEN_LABELS: Counter = Counter()


def main() -> None:
    """Create the HAREM NER dataset and upload it to the HF Hub."""
    # Download the HAREM dataset
    logger.info("Downloading HAREM dataset...")
    content = _download(URL)

    # Process the data
    logger.info("Processing HAREM data...")
    examples = _process_harem_data(content)

    logger.info(f"Total examples processed: {len(examples)}")
    logger.info("\nTag usage:")
    for tag, count in SEEN_TAGS.most_common():
        logger.info(f"  {tag:15s}: {count}")

    logger.info("\nLabel frequencies:")
    for label, count in SEEN_LABELS.most_common():
        logger.info(f"  {label:10s}: {count}")

    # Convert to DataFrame
    df = pd.DataFrame(examples)

    # Shuffle the dataset
    df = df.sample(frac=1, random_state=RANDOM_STATE).reset_index(drop=True)

    # Ensure we have enough examples for the desired splits
    total_needed = TRAIN_SIZE + VAL_SIZE + TEST_SIZE
    if len(df) < total_needed:
        logger.info(
            f"Warning: Only {len(df)} examples available, but {total_needed} needed"
        )
        # Adjust sizes proportionally
        ratio = len(df) / total_needed
        train_size = int(TRAIN_SIZE * ratio)
        val_size = int(VAL_SIZE * ratio)
        test_size = len(df) - train_size - val_size
    else:
        train_size = TRAIN_SIZE
        val_size = VAL_SIZE
        test_size = TEST_SIZE

    # Create splits
    train_df = df.iloc[:train_size].reset_index(drop=True)
    val_df = df.iloc[train_size : train_size + val_size].reset_index(drop=True)
    test_df = df.iloc[
        train_size + val_size : train_size + val_size + test_size
    ].reset_index(drop=True)

    logger.info("\nDataset splits:")
    logger.info(f"  Train: {len(train_df)} examples")
    logger.info(f"  Validation: {len(val_df)} examples")
    logger.info(f"  Test: {len(test_df)} examples")

    # Convert labels from strings to IDs for consistency with Spanish script
    def convert_labels_to_ids(labels: list[str]) -> list[int]:
        return [LABEL2ID[label] for label in labels]

    train_df["labels"] = train_df["labels"].apply(convert_labels_to_ids)
    val_df["labels"] = val_df["labels"].apply(convert_labels_to_ids)
    test_df["labels"] = test_df["labels"].apply(convert_labels_to_ids)

    # Convert back to strings for final dataset (matching Spanish script format)
    train_df["labels"] = train_df["labels"].apply(
        lambda ids: [ID2LABEL[id] for id in ids]
    )
    val_df["labels"] = val_df["labels"].apply(lambda ids: [ID2LABEL[id] for id in ids])
    test_df["labels"] = test_df["labels"].apply(
        lambda ids: [ID2LABEL[id] for id in ids]
    )

    # Create dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/harem"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    logger.info(f"\nUploading dataset to {dataset_id}...")
    dataset.push_to_hub(dataset_id, private=True)
    logger.info("Dataset uploaded successfully!")


def _download(url: str) -> str:
    """Download content from URL with proper encoding.

    Args:
        url: The URL to download from.

    Returns:
        The decoded content as a string.
    """
    with urllib.request.urlopen(url) as response:
        return response.read().decode("iso-8859-1")


def _parse_doc(doc: str) -> tuple[list[str], list[int]] | None:
    """Parse a single HAREM document and return tokens and labels (BIO format).

    Args:
        doc: The document string to parse.

    Returns:
        A tuple of (tokens, labels) if the document is valid, otherwise None.
    """
    origem_match = re.search(r"<ORIGEM>\s*(\w+)\s*</ORIGEM>", doc)
    if not origem_match or origem_match.group(1).upper() != "PT":
        return None

    text_match = re.search(r"<TEXTO>(.*?)</TEXTO>", doc, flags=re.S)
    if not text_match:
        return None
    text = text_match.group(1)

    tokens: list[str] = []
    labels: list[int] = []
    stack: list[str] = []
    previous_entity_type: str | None = None
    in_entity = False

    pos = 0
    for tag in TAG_RE.finditer(text):
        pre = text[pos : tag.start()]
        for tok in TOKEN_RE.findall(pre):
            if not stack:
                label = "O"
                previous_entity_type = None
                in_entity = False
            else:
                current_type = TAG2LABEL.get(stack[-1], "MISC")
                if not in_entity or current_type != previous_entity_type:
                    label = f"B-{current_type}"
                    in_entity = True
                else:
                    label = f"I-{current_type}"
                previous_entity_type = current_type

            tokens.append(tok)
            labels.append(LABEL2ID[label])
            SEEN_LABELS[label] += 1

        pos = tag.end()
        closing, name = tag.group(1), tag.group(2)
        SEEN_TAGS[name] += 1

        if name not in TAG2LABEL:
            logger.info(f"Warning: Unknown tag <{name}>")

        if closing:
            if stack and stack[-1] == name:
                stack.pop()
            # Reset entity tracking if closed
            in_entity = False
            previous_entity_type = None
        else:
            stack.append(name)
            in_entity = False  # next token should be B-

    tail = text[pos:]
    for tok in TOKEN_RE.findall(tail):
        if not stack:
            label = "O"
            previous_entity_type = None
            in_entity = False
        else:
            current_type = TAG2LABEL.get(stack[-1], "MISC")
            if not in_entity or current_type != previous_entity_type:
                label = f"B-{current_type}"
                in_entity = True
            else:
                label = f"I-{current_type}"
            previous_entity_type = current_type

        tokens.append(tok)
        labels.append(LABEL2ID[label])
        SEEN_LABELS[label] += 1

    return tokens, labels


def _reconstruct_text(tokens: list[str]) -> str:
    """Reconstruct text from tokens, preserving original spacing.

    Args:
        tokens: List of tokens.

    Returns:
        The reconstructed text as a string.
    """
    if not tokens:
        return ""

    result: list[str] = []
    for i, token in enumerate(tokens):
        if i == 0:
            # First token always gets added as-is
            result.append(token)
        elif re.match(r"[^\w\s]", token):
            # Punctuation - attach to previous token without space
            result.append(token)
        else:
            # Regular word - add space before
            result.append(" " + token)

    return "".join(result)


def _split_into_sentences(
    tokens: list[str], labels: list[int]
) -> list[tuple[list[str], list[int]]]:
    """Split tokens and labels into sentences.

    Args:
        tokens: List of tokens.
        labels: List of label IDs corresponding to tokens.

    Returns:
        List of (tokens, labels) tuples for each sentence.
    """
    sentences = []
    i = 0
    while i < len(tokens):
        current_tokens = []
        current_labels = []

        while i < len(tokens):
            current_tokens.append(tokens[i])
            current_labels.append(labels[i])

            if SENTENCE_RE.search(tokens[i]):
                i += 1
                # absorb following I-XXX tokens
                while i < len(tokens) and ID2LABEL[labels[i]].startswith("I-"):
                    current_tokens.append(tokens[i])
                    current_labels.append(labels[i])
                    i += 1
                break  # break inner loop

            i += 1

        sentences.append((current_tokens, current_labels))

    return sentences


def _process_harem_data(raw: str) -> list[dict]:
    """Process raw HAREM data into structured format.

    Args:
        raw: Raw string containing HAREM data.

    Returns:
        List of dicts, each with 'tokens', 'labels', and 'text' for a sentence.
    """
    docs = raw.split("<DOC>")
    examples = []

    for doc in docs:
        parsed = _parse_doc(doc)
        if parsed:
            tokens, labels = parsed

            # Split into sentences
            sentences = _split_into_sentences(tokens, labels)

            # Create examples for each sentence
            for sent_tokens, sent_labels in sentences:
                if len(sent_tokens) > 0:  # Only add non-empty sentences
                    # Convert labels back to strings
                    label_strings = [ID2LABEL[label] for label in sent_labels]

                    examples.append(
                        {
                            "tokens": sent_tokens,
                            "labels": label_strings,
                            "text": _reconstruct_text(sent_tokens),
                        }
                    )

    return examples


if __name__ == "__main__":
    main()



"""All Czech dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import CZECH
from ..tasks import COMMON_SENSE, KNOW, LA, NER, RC, SENT, SUMM

### Official datasets ###

CSFD_SENTIMENT_CONFIG = DatasetConfig(
    name="csfd-sentiment",
    pretty_name="CSFD Sentiment",
    source="EuroEval/csfd-sentiment-mini",
    task=SENT,
    languages=[CZECH],
)

CS_GEC_CONFIG = DatasetConfig(
    name="cs-gec",
    pretty_name="CS-GEC",
    source="EuroEval/cs-gec-mini",
    task=LA,
    languages=[CZECH],
)

PONER_CONFIG = DatasetConfig(
    name="poner",
    pretty_name="PoNER",
    source="EuroEval/poner-mini",
    task=NER,
    languages=[CZECH],
)

SQAD_CONFIG = DatasetConfig(
    name="sqad",
    pretty_name="SQAD",
    source="EuroEval/sqad-mini",
    task=RC,
    languages=[CZECH],
)

CZECH_NEWS_CONFIG = DatasetConfig(
    name="czech-news",
    pretty_name="Czech News",
    source="EuroEval/czech-news-mini",
    task=SUMM,
    languages=[CZECH],
)

UMIMETO_QA_CONFIG = DatasetConfig(
    name="umimeto-qa",
    pretty_name="Umimeto QA",
    source="EuroEval/umimeto-qa",
    task=KNOW,
    languages=[CZECH],
)

HELLASWAG_CS_CONFIG = DatasetConfig(
    name="hellaswag-cs",
    pretty_name="HellaSwag-cs",
    source="EuroEval/hellaswag-cs-mini",
    task=COMMON_SENSE,
    languages=[CZECH],
)


### Unofficial datasets ###

SCALA_CS_CONFIG = DatasetConfig(
    name="scala-cs",
    pretty_name="ScaLA-cs",
    source="EuroEval/scala-cs",
    task=LA,
    languages=[CZECH],
    unofficial=True,
)



"""Templates for the Named Entity Recognition task."""

import typing as t

from ..data_models import PromptConfig
from ..languages import (
    ALBANIAN,
    BOSNIAN,
    BULGARIAN,
    CATALAN,
    CROATIAN,
    CZECH,
    DANISH,
    DUTCH,
    ENGLISH,
    ESTONIAN,
    FAROESE,
    FINNISH,
    FRENCH,
    GERMAN,
    GREEK,
    HUNGARIAN,
    ICELANDIC,
    ITALIAN,
    LATVIAN,
    LITHUANIAN,
    NORWEGIAN,
    NORWEGIAN_BOKMÅL,
    NORWEGIAN_NYNORSK,
    POLISH,
    PORTUGUESE,
    ROMANIAN,
    SERBIAN,
    SLOVAK,
    SLOVENE,
    SPANISH,
    SWEDISH,
    UKRAINIAN,
)

if t.TYPE_CHECKING:
    from ..languages import Language


NER_TEMPLATES: dict["Language", PromptConfig] = {
    ALBANIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "person",
            "i-per": "person",
            "b-loc": "vendndodhje",
            "i-loc": "vendndodhje",
            "b-org": "organizatë",
            "i-org": "organizatë",
            "b-misc": "të ndryshme",
            "i-misc": "të ndryshme",
        },
        default_prompt_prefix="Më poshtë janë fjali dhe fjalorë JSON me entitetet e "
        "emërtuara që shfaqen në fjalinë e dhënë.",
        default_prompt_template="Fjali: {text}\nEntitete të emërtuara: {label}",
        default_instruction_prompt="Fjali: {text}\n\nIdentifikoni entitetet e "
        "emërtuara në fjali. Duhet t’i jepni ato si një fjalor JSON me çelësat "
        "{labels_str}. Vlerat duhet të jenë lista të entiteteve të emërtuara të atij "
        "lloji, saktësisht ashtu siç shfaqen në fjali.",
    ),
    BOSNIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "osoba",
            "i-per": "osoba",
            "b-loc": "mjesto",
            "i-loc": "mjesto",
            "b-org": "organizacija",
            "i-org": "organizacija",
            "b-misc": "razno",
            "i-misc": "razno",
        },
        default_prompt_prefix="Slijede rečenice i JSON riječnici s imenovanim "
        "entitetima koji se pojavljuju u rečenicama.",
        default_prompt_template="Rečenica: {text}\nImenovani entiteti: {label}",
        default_instruction_prompt="Rečenica: {text}\n\nIdentificirajte imenovane "
        "entitete u rečenici. Prikažite ih kao JSON riječnik s ključevima "
        "{labels_str}. Vrijednosti trebaju biti popisi imenovanih entiteta "
        "navedenog tipa, točno kako se pojavljuju u rečenici.",
    ),
    BULGARIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "лице",
            "i-per": "лице",
            "b-loc": "място",
            "i-loc": "място",
            "b-org": "организация",
            "i-org": "организация",
            "b-misc": "разни",
            "i-misc": "разни",
        },
        default_prompt_prefix="По-долу са изречения и JSON речници с именуваните "
        "обекти, които се срещат в дадените изречения.",
        default_prompt_template="Изречение: {text}\nИменувани обекти: {label}",
        default_instruction_prompt="Изречение: {text}\n\nИдентифицирайте именуваните "
        "обекти в изречението. Трябва да изведете това като JSON речник с ключовете "
        "{labels_str}. Стойностите трябва да бъдат списъци на именуваните обекти от "
        "този тип, точно както се появяват в изречението.",
    ),
    CATALAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "persona",
            "i-per": "persona",
            "b-loc": "lloc",
            "i-loc": "lloc",
            "b-org": "organització",
            "i-org": "organització",
            "b-misc": "miscel·lània",
            "i-misc": "miscel·lània",
        },
        default_prompt_prefix="Aquestes són frases i diccionaris JSON amb els noms "
        "que apareixen en les frases.",
        default_prompt_template="Frase: {text}\nEntitats anomenades: {label}",
        default_instruction_prompt="Frase: {text}\n\nIdentifiqueu les entitats "
        "anomenades en la frase. Mostreu-les com a diccionari JSON amb les claus "
        "{labels_str}. Els valors han de ser els llistats de les entitats anomenades "
        "del tipus, tal com apareixen en la frase.",
    ),
    CROATIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "osoba",
            "i-per": "osoba",
            "b-loc": "mjesto",
            "i-loc": "mjesto",
            "b-org": "organizacija",
            "i-org": "organizacija",
            "b-misc": "razno",
            "i-misc": "razno",
        },
        default_prompt_prefix="Sljedeće su rečenice i JSON rječnici s imenicama koje "
        "se pojavljuju u rečenicama.",
        default_prompt_template="Rečenica: {text}\nImenovane entiteti: {label}",
        default_instruction_prompt="Rečenica: {text}\n\nIdentificirajte imenovane "
        "entitete u rečenici. Prikažite ih kao JSON rječnik s ključevima "
        "{labels_str}. Vrijednosti trebaju biti popisi imenovanih entiteta "
        "navedenog tipa, točno kako se pojavljuju u rečenici.",
    ),
    CZECH: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "osoba",
            "i-per": "osoba",
            "b-loc": "místo",
            "i-loc": "místo",
            "b-org": "organizace",
            "i-org": "organizace",
            "b-misc": "různé",
            "i-misc": "různé",
        },
        default_prompt_prefix="Následující jsou věty a JSON slovníky s pojmenovanými "
        "entitami, které se v dané větě vyskytují.",
        default_prompt_template="Věta: {text}\nPojmenované entity: {label}",
        default_instruction_prompt="Věta: {text}\n\nIdentifikujte pojmenované entity "
        "ve větě. Měli byste to vypsat jako JSON slovník s klíči {labels_str}. "
        "Hodnoty by měly být seznamy pojmenovaných entit tohoto typu, přesně tak, "
        "jak se objevují ve větě.",
    ),
    DANISH: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "person",
            "i-per": "person",
            "b-loc": "sted",
            "i-loc": "sted",
            "b-org": "organisation",
            "i-org": "organisation",
            "b-misc": "diverse",
            "i-misc": "diverse",
        },
        default_prompt_prefix="Følgende er sætninger og JSON-ordbøger med de navngivne "
        "enheder, som forekommer i den givne sætning.",
        default_prompt_template="Sætning: {text}\nNavngivne enheder: {label}",
        default_instruction_prompt="Sætning: {text}\n\nIdentificér de navngivne "
        "enheder i sætningen. Du skal outputte dette som en JSON-ordbog med nøglerne "
        "{labels_str}. Værdierne skal være lister over de navngivne enheder af den "
        "type, præcis som de forekommer i sætningen.",
    ),
    GERMAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "person",
            "i-per": "person",
            "b-loc": "ort",
            "i-loc": "ort",
            "b-org": "organisation",
            "i-org": "organisation",
            "b-misc": "verschiedenes",
            "i-misc": "verschiedenes",
        },
        default_prompt_prefix="Es folgen Sätze und JSON-Wörterbücher mit den benannten "
        "Entitäten, die in der angegebenen Phrase vorkommen.",
        default_prompt_template="Satz: {text}\nBenannte Entitäten: {label}",
        default_instruction_prompt="Satz: {text}\n\nIdentifizieren Sie die benannten "
        "Entitäten im Satz. Sie sollten dies als JSON-Wörterbuch mit den "
        "Schlüsseln {labels_str} ausgeben. Die Werte sollten Listen der "
        "benannten Entitäten dieses Typs sein, genau wie sie im Satz erscheinen.",
    ),
    GREEK: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "πρόσωπο",
            "i-per": "πρόσωπο",
            "b-loc": "τοποθεσία",
            "i-loc": "τοποθεσία",
            "b-org": "οργανισμός",
            "i-org": "οργανισμός",
            "b-misc": "διάφορα",
            "i-misc": "διάφορα",
        },
        default_prompt_prefix="Ακολουθούν προτάσεις και λεξικά JSON με τις "
        "ονομαστικές οντότητες που εμφανίζονται στην δεδομένη πρόταση.",
        default_prompt_template="Πρόταση: {text}\nΟνομαστικές οντότητες: {label}",
        default_instruction_prompt="Πρόταση: {text}\n\nΑναγνωρίστε τις ονομαστικές "
        "οντότητες στην πρόταση. Θα πρέπει να παράγετε αυτό ως λεξικό JSON με "
        "κλειδιά {labels_str}. Οι τιμές πρέπει να είναι λίστες των ονομαστικών "
        "οντοτήτων αυτού του τύπου, ακριβώς όπως εμφανίζονται στην πρόταση.",
    ),
    HUNGARIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "személy",
            "i-per": "személy",
            "b-loc": "helyszín",
            "i-loc": "helyszín",
            "b-org": "szervezet",
            "i-org": "szervezet",
            "b-misc": "egyéb",
            "i-misc": "egyéb",
        },
        default_prompt_prefix="Az alábbiakban mondatok és JSON szótárak találhatók\n"
        "az adott mondatokban előforduló névjegyzékkel.",
        default_prompt_template="Mondat: {text}\nNévjegyzék: {label}",
        default_instruction_prompt="Mondat: {text}\n\nNevezze meg a mondatban "
        "szereplő neveket. JSON szótárként adja meg a {labels_str} kulcsszavakat. "
        "Az értékek a mondatban szereplő névjegyzékek listái legyenek, pontosan úgy, "
        "ahogyan megjelennek.",
    ),
    ENGLISH: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "person",
            "i-per": "person",
            "b-loc": "location",
            "i-loc": "location",
            "b-org": "organization",
            "i-org": "organization",
            "b-misc": "miscellaneous",
            "i-misc": "miscellaneous",
        },
        default_prompt_prefix="Below are sentences and JSON dictionaries with the "
        "named entities that occur in the given sentence.",
        default_prompt_template="Sentence: {text}\nNamed entities: {label}",
        default_instruction_prompt="Sentence: {text}\n\nIdentify the named entities in "
        "the sentence. You should output this as a JSON dictionary with the keys being "
        "{labels_str}. The values should be lists of the named entities of that "
        "type, exactly as they appear in the sentence.",
    ),
    SPANISH: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "persona",
            "i-per": "persona",
            "b-loc": "lugar",
            "i-loc": "lugar",
            "b-org": "organización",
            "i-org": "organización",
            "b-misc": "misceláneo",
            "i-misc": "misceláneo",
        },
        default_prompt_prefix="Lo siguiente son oraciones y diccionarios JSON con las "
        "entidades nombradas que aparecen en la oración dada.",
        default_prompt_template="Oración: {text}\nEntidades nombradas: {label}",
        default_instruction_prompt="Oración: {text}\n\nIdentifica las entidades "
        "nombradas en la oración. Debes producir esto como un diccionario JSON con las "
        "claves {labels_str}. Los valores deben ser listas de las "
        "entidades nombradas de ese tipo, exactamente como aparecen en la oración.",
    ),
    ESTONIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "inimene",
            "i-per": "inimene",
            "b-loc": "asukoht",
            "i-loc": "asukoht",
            "b-org": "organisatsioon",
            "i-org": "organisatsioon",
            "b-misc": "muu",
            "i-misc": "muu",
        },
        default_prompt_prefix="Allpool on laused ja JSON-sõnastikud, mis sisaldavad "
        "antud lauses esinevaid nimetatud üksuseid.",
        default_prompt_template="Lause: {text}\nNimetatud üksused: {label}",
        default_instruction_prompt="Lause: {text}\n\nTuvasta lauses "
        "nimetatud üksused. Väljund peaks olema JSON-sõnastik, "
        "mille võtmed on {labels_str}. Väärtused peaksid olema kindlat tüüpi nimetatud "
        "üksuste loendid, täpselt nii nagu need lauses esinevad.",
    ),
    PORTUGUESE: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "pessoa",
            "i-per": "pessoa",
            "b-loc": "local",
            "i-loc": "local",
            "b-org": "organização",
            "i-org": "organização",
            "b-misc": "diverso",
            "i-misc": "diverso",
        },
        default_prompt_prefix="Seguem-se frases e dicionários JSON com as entidades "
        "mencionadas presentes na frase indicada.",
        default_prompt_template="Frase: {text}\nEntidades mencionadas: {label}",
        default_instruction_prompt="Frase: {text}\n\nIdentifica as entidades "
        "mencionadas na frase. Deves devolver um dicionário JSON com as chaves "
        "{labels_str}. Os valores devem ser listas contendo as entidades "
        "mencionadas desse tipo, tal como ocorrem na frase.",
    ),
    FINNISH: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "henkilö",
            "i-per": "henkilö",
            "b-loc": "paikka",
            "i-loc": "paikka",
            "b-org": "organisaatio",
            "i-org": "organisaatio",
            "b-misc": "muut",
            "i-misc": "muut",
        },
        default_prompt_prefix="Seuraavassa on lauseita ja JSON-sanakirjoja, jotka "
        "sisältävät annetussa lauseessa esiintyvät nimetyt entiteetit.",
        default_prompt_template="Lause: {text}\nNimetyt entiteetit: {label}",
        default_instruction_prompt="Lause: {text}\n\nTunnista lauseessa olevat "
        "entiteetit. Tulosta ne JSON-sanakirjana, jonka avaimet ovat {labels_str}. "
        "Arvojen tulee olla listoja kyseisen tyypin nimetyistä entiteeteistä "
        "täsmälleen siinä muodossa kuin ne esiintyvät lauseessa.",
    ),
    FAROESE: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "persónur",
            "i-per": "persónur",
            "b-loc": "staður",
            "i-loc": "staður",
            "b-org": "felagsskapur",
            "i-org": "felagsskapur",
            "b-misc": "ymiskt",
            "i-misc": "ymiskt",
        },
        default_prompt_prefix="Her eru nakrir setningar og nakrar JSON orðabøkur við "
        "nevndar eindir, sum eru í setningunum.",
        default_prompt_template="Setningur: {text}\nNevndar eindir: {label}",
        default_instruction_prompt="Setningur: {text}\n\nGreindu nevndu einingarnar í "
        "setningunni. Þú ættir að skila þessu sem JSON orðabók með lyklunum "
        "{labels_str}. Gildin ættu að vera listi yfir nevndu einingarnar af "
        "þeirri gerð, nákvæmlega eins og þær koma fram í setningunni.",
    ),
    FRENCH: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "personne",
            "i-per": "personne",
            "b-loc": "lieu",
            "i-loc": "lieu",
            "b-org": "organisation",
            "i-org": "organisation",
            "b-misc": "divers",
            "i-misc": "divers",
        },
        default_prompt_prefix="Vous trouverez ci-dessous des phrases et des "
        "dictionnaires JSON avec les entités nommées qui apparaissent dans la "
        "phrase donnée.",
        default_prompt_template="Sentence: {text}\nEntités nommées: {label}",
        default_instruction_prompt="Sentence: {text}\n\nIdentifiez les entités nommées "
        "dans la phrase. Vous devez produire ceci sous forme de dictionnaire JSON "
        "avec les clés {labels_str}. Les valeurs doivent être des listes des "
        "entités nommées de ce type, exactement comme elles apparaissent dans "
        "la phrase.",
    ),
    ICELANDIC: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "einstaklingur",
            "i-per": "einstaklingur",
            "b-loc": "staðsetning",
            "i-loc": "staðsetning",
            "b-org": "stofnun",
            "i-org": "stofnun",
            "b-misc": "ýmislegt",
            "i-misc": "ýmislegt",
        },
        default_prompt_prefix="Eftirfarandi eru setningar ásamt JSON lyklum með "
        "nefndum einingum sem koma fyrir í setningunum.",
        default_prompt_template="Setning: {text}\nNafneiningar: {label}",
        default_instruction_prompt="Setning: {text}\n\nGreindu nefndu einingarnar í "
        "setningunni. Þú ættir að skila þessu sem JSON orðabók með lyklunum "
        "{labels_str}. Gildin ættu að vera listi yfir nefndu "
        "einingarnar af þeirri gerð, nákvæmlega eins og þær koma fram í "
        "setningunni.",
    ),
    ITALIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "persona",
            "i-per": "persona",
            "b-loc": "posizione",
            "i-loc": "posizione",
            "b-org": "organizzazione",
            "i-org": "organizzazione",
            "b-misc": "varie",
            "i-misc": "varie",
        },
        default_prompt_prefix="Di seguito sono riportate le frasi e i dizionari JSON "
        "con le entità denominate presenti nella frase data.",
        default_prompt_template="Frase: {text}\nEntità denominate: {label}",
        default_instruction_prompt="Frase: {text}\n\nIdentificare le entità nominate "
        "nella frase. Il risultato dovrebbe essere un dizionario JSON con le chiavi "
        "{labels_str}. I valori devono essere elenchi di entità "
        "nominate di quel tipo, esattamente come appaiono nella frase.",
    ),
    LITHUANIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "asmuo",
            "i-per": "asmuo",
            "b-loc": "vieta",
            "i-loc": "vieta",
            "b-org": "organizacija",
            "i-org": "organizacija",
            "b-misc": "kita",
            "i-misc": "kita",
        },
        default_prompt_prefix="Toliau pateikti sakiniai ir JSON žodynai su vardiniais "
        "vienetais, kurie pateikiame sakinyje.",
        default_prompt_template="Sakinys: {text}\nVardiniai vienetai: {label}",
        default_instruction_prompt="Sakinys: {text}\n\nIdentifikuokite vardinius "
        "vienetus sakinyje. Turėtumėte pateikti tai kaip JSON žodyną su raktais "
        "{labels_str}. Reikšmės turi būti to tipo vardinių vienetų sąrašai, "
        "tiksliai taip, kaip jie rodomi sakinyje.",
    ),
    LATVIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "persona",
            "i-per": "persona",
            "b-loc": "vieta",
            "i-loc": "vieta",
            "b-org": "organizācija",
            "i-org": "organizācija",
            "b-misc": "dažādi",
            "i-misc": "dažādi",
        },
        default_prompt_prefix="Tālāk ir teikumi un JSON vārdnīcas ar nosauktajiem "
        "objektiem, kas parādās dotajā teikumā.",
        default_prompt_template="Teikums: {text}\nNosauktie objekti: {label}",
        default_instruction_prompt="Teikums: {text}\n\n"
        "Identificējiet nosauktos objektus "
        "teikumā. Jums jāizvada šī informācija kā JSON vārdnīcu ar atslēgām "
        "{labels_str}. Vērtībām jābūt šī tipa nosaukto objektu sarakstiem, "
        "tieši tā, kā tie parādās teikumā.",
    ),
    NORWEGIAN_BOKMÅL: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "person",
            "i-per": "person",
            "b-loc": "sted",
            "i-loc": "sted",
            "b-org": "organisasjon",
            "i-org": "organisasjon",
            "b-misc": "diverse",
            "i-misc": "diverse",
        },
        default_prompt_prefix="Følgende er fraser og JSON-ordbøker med de navngitte "
        "enhetene som forekommer i den gitte frasen.",
        default_prompt_template="Frase: {text}\nNavngitte enheter: {label}",
        default_instruction_prompt="Frase: {text}\n\nIdentifiser de navngitte "
        "enhetene i frasen. Du bør outputte dette som en JSON-ordbok med nøklene "
        "{labels_str}. Verdiene skal være lister over de navngitte enhetene av den "
        "typen, akkurat som de vises i frasen.",
    ),
    DUTCH: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "persoon",
            "i-per": "persoon",
            "b-loc": "locatie",
            "i-loc": "locatie",
            "b-org": "organisatie",
            "i-org": "organisatie",
            "b-misc": "diversen",
            "i-misc": "diversen",
        },
        default_prompt_prefix="Hieronder staan zinnen en JSON woordenboeken met de "
        "genoemde entiteiten die voorkomen in de gegeven zin.",
        default_prompt_template="Zin: {text}\nGenoemde entiteiten: {label}",
        default_instruction_prompt="Zin: {text}\n\nIdentificeer de genoemde entiteiten "
        "in de zin. Je moet dit uitvoeren als een JSON-woordenboek met de sleutels "
        "{labels_str}. De waarden moeten lijsten zijn van de "
        "genoemde entiteiten van dat type, precies zoals ze voorkomen in de zin.",
    ),
    NORWEGIAN_NYNORSK: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "person",
            "i-per": "person",
            "b-loc": "sted",
            "i-loc": "sted",
            "b-org": "organisasjon",
            "i-org": "organisasjon",
            "b-misc": "diverse",
            "i-misc": "diverse",
        },
        default_prompt_prefix="Følgende er fraser og JSON-ordbøker med de navngitte "
        "enhetene som forekommer i den gitte frasen.",
        default_prompt_template="Frase: {text}\nNavngitte enheter: {label}",
        default_instruction_prompt="Frase: {text}\n\nIdentifiser de navngitte enhetene "
        "i frasen. Du bør outputte dette som en JSON-ordbok med nøklene {labels_str}."
        "Verdiene skal være lister over de navngitte enhetene "
        "av den typen, akkurat som de vises i frasen.",
    ),
    NORWEGIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "person",
            "i-per": "person",
            "b-loc": "sted",
            "i-loc": "sted",
            "b-org": "organisasjon",
            "i-org": "organisasjon",
            "b-misc": "diverse",
            "i-misc": "diverse",
        },
        default_prompt_prefix="Følgende er fraser og JSON-ordbøker med de navngitte "
        "enhetene som forekommer i den gitte frasen.",
        default_prompt_template="Frase: {text}\nNavngitte enheter: {label}",
        default_instruction_prompt="Frase: {text}\n\nIdentifiser de navngitte enhetene "
        "i frasen. Du bør outputte dette som en JSON-ordbok med nøklene {labels_str}."
        "Verdiene skal være lister over de navngitte enhetene "
        "av den typen, akkurat som de vises i frasen.",
    ),
    ROMANIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "persoană",
            "i-per": "persoană",
            "b-loc": "locație",
            "i-loc": "locație",
            "b-org": "organizație",
            "i-org": "organizație",
            "b-misc": "diverse",
            "i-misc": "diverse",
        },
        default_prompt_prefix="Mai jos sunt propoziții și dicționare JSON cu "
        "entitățile numite care apar în propoziția dată.",
        default_prompt_template="Propoziție: {text}\nEntități numite: {label}",
        default_instruction_prompt="Propoziție: {text}\n\nIdentifică entitățile "
        "numite din propoziție. Ar trebui să le enumeri ca un dicționar JSON cu "
        "cheile {labels_str}. Valorile cheilor ar trebui să fie liste de entități "
        "numite de tipul respectiv, exact cum apar în propoziție.",
    ),
    POLISH: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "osoba",
            "i-per": "osoba",
            "b-loc": "miejsce",
            "i-loc": "miejsce",
            "b-org": "organizacja",
            "i-org": "organizacja",
            "b-misc": "inne",
            "i-misc": "inne",
        },
        default_prompt_prefix="Poniżej znajdują się zdania i słowniki JSON "
        "z jednostkami nazewniczymi, które występują w danym zdaniu.",
        default_prompt_template="Zdanie: {text}\nJednostki nazewnicze: {label}",
        default_instruction_prompt="Zdanie: {text}\n\nZidentyfikuj jednostki "
        "nazewnicze w zdaniu. Wypisz je jako słownik JSON z kluczami "
        "{labels_str}. Wartości odpowiadające kluczom powinny być listami jednostek "
        "nazewniczych danego typu, dokładnie tak, jak pojawiają się w zdaniu.",
    ),
    SLOVAK: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "osoba",
            "i-per": "osoba",
            "b-loc": "miesto",
            "i-loc": "miesto",
            "b-org": "organizácia",
            "i-org": "organizácia",
            "b-misc": "rôzne",
            "i-misc": "rôzne",
        },
        default_prompt_prefix="Nasledujúce sú vety a JSON-objekty s pomenovanými "
        "entitami, ktoré sa nachádzajú v danej vete.",
        default_prompt_template="Veta: {text}\nPomenované entity: {label}",
        default_instruction_prompt="Veta: {text}\n\nIdentifikujte pomenované "
        "entity vo vete. Výstup by mal byť vo forme JSON-objektu s kľúčmi "
        "{labels_str}. Hodnoty by mali byť zoznamy pomenovaných entít danej "
        "kategórie, presne tak, ako sa vyskytujú vo vete.",
    ),
    SLOVENE: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "oseba",
            "i-per": "oseba",
            "b-loc": "kraj",
            "i-loc": "kraj",
            "b-org": "organizacija",
            "i-org": "organizacija",
            "b-misc": "razno",
            "i-misc": "razno",
        },
        default_prompt_prefix="Naslednje so povedi in JSON slovarji z poimenovanimi "
        "entitetami, ki se pojavijo v dani povedi.",
        default_prompt_template="Poved: {text}\nPoimenovane entitete: {label}",
        default_instruction_prompt="Poved: {text}\n\nIdentificirajte poimenovane "
        "entitete v povedi. To morate izpisati kot JSON slovar s ključi {labels_str}. "
        "Vrednosti morajo biti seznami poimenovanih entitet te kategorije, tako kot "
        "se pojavijo v povedi.",
    ),
    SERBIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "osoba",
            "i-per": "osoba",
            "b-loc": "mesto",
            "i-loc": "mesto",
            "b-org": "organizacija",
            "i-org": "organizacija",
            "b-misc": "razno",
            "i-misc": "razno",
        },
        default_prompt_prefix="Sledeće su rečenice i JSON rečnici sa imenovanim "
        "entitetima koji se pojavljuju u datoj rečenici.",
        default_prompt_template="Rečenica: {text}\nImenovani entiteti: {label}",
        default_instruction_prompt="Rečenica: {text}\n\nIdentifikujte imenovane "
        "entitete u rečenici. Trebalo bi da ovo ispišete kao JSON rečnik sa ključevima "
        "{labels_str}. Vrednosti treba da budu liste imenovanih entiteta te "
        "kategorije, tačno onako kako se pojavljuju u rečenici.",
    ),
    SWEDISH: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "person",
            "i-per": "person",
            "b-loc": "plats",
            "i-loc": "plats",
            "b-org": "organisation",
            "i-org": "organisation",
            "b-misc": "diverse",
            "i-misc": "diverse",
        },
        default_prompt_prefix="Följande är meningar och JSON-ordböcker med de "
        "namngivna enheter som förekommer i den givna meningen.",
        default_prompt_template="Mening: {text}\nNamngivna entiteter: {label}",
        default_instruction_prompt="Mening: {text}\n\nIdentifiera de namngivna "
        "enheterna i meningen. Du ska outputta detta som en JSON-ordbok med nycklarna "
        "{labels_str}. Värdena ska vara listor över de namngivna enheterna av den "
        "typen, precis som de förekommer i meningen.",
    ),
    UKRAINIAN: PromptConfig(
        default_prompt_label_mapping={
            "b-per": "особа",
            "i-per": "особа",
            "b-loc": "місце",
            "i-loc": "місце",
            "b-org": "організація",
            "i-org": "організація",
            "b-misc": "різне",
            "i-misc": "різне",
        },
        default_prompt_prefix="Нижче наведені речення та JSON-словники з іменованими "
        "сутностями, які присутні у даному реченні.",
        default_prompt_template="Речення: {text}\nІменовані сутності: {label}",
        default_instruction_prompt="Речення: {text}\n\n"
        "Ідентифікуйте іменовані сутності у "
        "реченні. Ви повинні вивести це як JSON-словник з ключами {labels_str}. "
        "Значення мають бути списками іменованих сутностей цього типу, точно "
        "такими, як вони з'являються у реченні.",
    ),
}



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the ARC-mini datasets and upload them to the HF Hub."""

from collections import Counter

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi

from .constants import (
    MAX_NUM_CHARS_IN_INSTRUCTION,
    MAX_NUM_CHARS_IN_OPTION,
    MAX_REPETITIONS,
    MIN_NUM_CHARS_IN_INSTRUCTION,
    MIN_NUM_CHARS_IN_OPTION,
)


def main() -> None:
    """Create the ARC-mini datasets and upload them to the HF Hub."""
    # Download the dataset
    repo_id = "mideind/icelandic-arc-challenge"
    dataset = load_dataset(path=repo_id, token=True)
    assert isinstance(dataset, DatasetDict)

    def prepare_dataframe(dataset: Dataset) -> pd.DataFrame:
        """Prepare a dataframe from a dataset.

        Args:
            dataset:
                The dataset to prepare.

        Returns:
            A dataframe with the prepared dataset.
        """
        df = dataset.to_pandas()
        assert isinstance(df, pd.DataFrame)

        # Rename the columns
        df.rename(columns=dict(answerKey="label", question="instruction"), inplace=True)

        # Remove samples where the `label` is not 'A', 'B', 'C' or 'D'
        df = df.loc[df.label.isin(["A", "B", "C", "D"])]
        df = df.loc[
            df.choices.map(lambda dct: tuple(dct["label"])) == ("A", "B", "C", "D")
        ]

        df["option_a"] = df.choices.map(lambda dct: dct["text"][0])
        df["option_b"] = df.choices.map(lambda dct: dct["text"][1])
        df["option_c"] = df.choices.map(lambda dct: dct["text"][2])
        df["option_d"] = df.choices.map(lambda dct: dct["text"][3])

        # Remove all samples with a null value of `option_a`, `option_b`,
        # `option_c` or `option_d`
        df = df.loc[
            df["option_a"].notnull()
            & df["option_b"].notnull()
            & df["option_c"].notnull()
            & df["option_d"].notnull()
        ]

        # Remove the samples with overly short or long texts
        df = df.loc[
            (df.instruction.str.len() >= MIN_NUM_CHARS_IN_INSTRUCTION)
            & (df.instruction.str.len() <= MAX_NUM_CHARS_IN_INSTRUCTION)
            & (df.option_a.str.len() >= MIN_NUM_CHARS_IN_OPTION)
            & (df.option_a.str.len() <= MAX_NUM_CHARS_IN_OPTION)
            & (df.option_b.str.len() >= MIN_NUM_CHARS_IN_OPTION)
            & (df.option_b.str.len() <= MAX_NUM_CHARS_IN_OPTION)
            & (df.option_c.str.len() >= MIN_NUM_CHARS_IN_OPTION)
            & (df.option_c.str.len() <= MAX_NUM_CHARS_IN_OPTION)
            & (df.option_d.str.len() >= MIN_NUM_CHARS_IN_OPTION)
            & (df.option_d.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        ]

        def is_repetitive(text: str) -> bool:
            """Return True if the text is repetitive."""
            max_repetitions = max(Counter(text.split()).values())
            return max_repetitions > MAX_REPETITIONS

        # Remove overly repetitive samples
        df = df[
            ~df.instruction.apply(is_repetitive)
            & ~df.option_a.apply(is_repetitive)
            & ~df.option_b.apply(is_repetitive)
            & ~df.option_c.apply(is_repetitive)
            & ~df.option_d.apply(is_repetitive)
        ]

        # Make a `text` column with all the options in it
        df["text"] = [
            row.instruction.replace("\n", " ").strip() + "\n"
            "Svarmöguleikar:\n"
            "a. " + row.option_a.replace("\n", " ").strip() + "\n"
            "b. " + row.option_b.replace("\n", " ").strip() + "\n"
            "c. " + row.option_c.replace("\n", " ").strip() + "\n"
            "d. " + row.option_d.replace("\n", " ").strip()
            for _, row in df.iterrows()
        ]

        # Make the `label` column case-consistent with the `text` column
        df.label = df.label.str.lower()

        # Only keep the `text` and `label` columns
        df = df.loc[["text", "label"]]

        # Remove duplicates
        df.drop_duplicates(inplace=True)
        df.reset_index(drop=True, inplace=True)

        return df

    # Create validation split
    val_size = 256
    val_df = prepare_dataframe(dataset=dataset["validation"]).sample(
        n=val_size, random_state=4242
    )

    # Create test split
    test_size = 1024
    test_df = prepare_dataframe(dataset=dataset["test"]).sample(
        n=test_size, random_state=4242
    )

    # Create train split
    train_size = 1024
    train_df = prepare_dataframe(dataset=dataset["train"]).sample(
        n=train_size, random_state=4242
    )

    # Reset the index
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/arc-is-mini"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the SQAD-mini Czech dataset and upload it to the HF Hub."""

import hashlib

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi

from .constants import (
    MAX_NUM_CHARS_IN_CONTEXT,
    MAX_NUM_CHARS_IN_QUESTION,
    MIN_NUM_CHARS_IN_CONTEXT,
    MIN_NUM_CHARS_IN_QUESTION,
)


def main() -> None:
    """Create the SQAD-mini Czech dataset and upload it to the HF Hub."""
    # Load the dataset
    dataset = load_dataset("CZLC/sqad_3.2_filtered")
    assert isinstance(dataset, DatasetDict)

    # Convert to DataFrames
    train_df = dataset["train"].to_pandas()
    test_df = dataset["test"].to_pandas()

    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Filter out data points with more than one answer
    train_df = train_df[train_df["answers"].apply(len) == 1]
    test_df = test_df[test_df["answers"].apply(len) == 1]

    # Filter based on context and question length
    train_df = train_df[
        train_df.context.str.len().between(
            MIN_NUM_CHARS_IN_CONTEXT, MAX_NUM_CHARS_IN_CONTEXT
        )
        & train_df.question.str.len().between(
            MIN_NUM_CHARS_IN_QUESTION, MAX_NUM_CHARS_IN_QUESTION
        )
    ]
    test_df = test_df[
        test_df.context.str.len().between(
            MIN_NUM_CHARS_IN_CONTEXT, MAX_NUM_CHARS_IN_CONTEXT
        )
        & test_df.question.str.len().between(
            MIN_NUM_CHARS_IN_QUESTION, MAX_NUM_CHARS_IN_QUESTION
        )
    ]

    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Extract answer start positions
    train_df["answer_start"] = train_df.apply(
        lambda row: row["context"].find(row["answers"][0]), axis=1
    )
    test_df["answer_start"] = test_df.apply(
        lambda row: row["context"].find(row["answers"][0]), axis=1
    )

    # Make the `answers` column a dictionary with the answer and the answer start
    train_df["answers"] = train_df.apply(
        lambda row: {
            "answer_start": [row["answer_start"]],
            "text": [row["answers"][0]],
        },
        axis=1,
    )
    test_df["answers"] = test_df.apply(
        lambda row: {
            "answer_start": [row["answer_start"]],
            "text": [row["answers"][0]],
        },
        axis=1,
    )

    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Define split sizes
    train_size = 1024
    val_size = 256
    test_size = 2048

    # Create new train and validation splits from the train split
    final_train_df = train_df.sample(n=train_size, random_state=4242)
    remaining_df = train_df.drop(final_train_df.index.tolist())
    final_val_df = remaining_df.sample(n=val_size, random_state=4242)

    # Use the existing test split
    final_test_df = test_df.sample(n=test_size, random_state=4242)

    # Reset indices
    final_train_df = final_train_df.reset_index(drop=True)
    final_val_df = final_val_df.reset_index(drop=True)
    final_test_df = final_test_df.reset_index(drop=True)

    # Add ID column
    final_train_df["id"] = [
        hashlib.md5(
            (row.title + row.context + row.question).encode("utf-8")
        ).hexdigest()
        for _, row in final_train_df.iterrows()
    ]
    final_val_df["id"] = [
        hashlib.md5(
            (row.title + row.context + row.question).encode("utf-8")
        ).hexdigest()
        for _, row in final_val_df.iterrows()
    ]
    final_test_df["id"] = [
        hashlib.md5(
            (row.title + row.context + row.question).encode("utf-8")
        ).hexdigest()
        for _, row in final_test_df.iterrows()
    ]

    # Check that the IDs are unique
    assert final_train_df.id.nunique() == len(final_train_df)
    assert final_val_df.id.nunique() == len(final_val_df)
    assert final_test_df.id.nunique() == len(final_test_df)

    # Create DatasetDict
    dataset_dict = DatasetDict(
        {
            "train": Dataset.from_pandas(final_train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(final_val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(final_test_df, split=Split.TEST),
        }
    )

    # Push to Hugging Face Hub
    dataset_id = "EuroEval/sqad-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset_dict.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



"""All Estonian dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import ESTONIAN
from ..tasks import COMMON_SENSE, EUROPEAN_VALUES, KNOW, LA, NER, RC, SENT, SUMM

### Official datasets ###

ESTONIAN_VALENCE_CONFIG = DatasetConfig(
    name="estonian-valence",
    pretty_name="Estonian Valence",
    source="EuroEval/estonian-valence",
    task=SENT,
    languages=[ESTONIAN],
)

GRAMMAR_ET_CONFIG = DatasetConfig(
    name="grammar-et",
    pretty_name="Grammar-et",
    source="EuroEval/grammar-et",
    task=LA,
    languages=[ESTONIAN],
)

ESTNER_CONFIG = DatasetConfig(
    name="estner",
    pretty_name="EstNER",
    source="EuroEval/estner-mini",
    task=NER,
    languages=[ESTONIAN],
)

MULTI_WIKI_QA_ET_CONFIG = DatasetConfig(
    name="multi-wiki-qa-et",
    pretty_name="MultiWikiQA-et",
    source="EuroEval/multi-wiki-qa-et-mini",
    task=RC,
    languages=[ESTONIAN],
)

ERR_NEWS_CONFIG = DatasetConfig(
    name="err-news",
    pretty_name="ERR News",
    source="EuroEval/err-news-mini",
    task=SUMM,
    languages=[ESTONIAN],
)

TRIVIA_ET_CONFIG = DatasetConfig(
    name="trivia-et",
    pretty_name="Trivia-et",
    source="EuroEval/trivia-et",
    task=KNOW,
    languages=[ESTONIAN],
)

WINOGRANDE_ET_CONFIG = DatasetConfig(
    name="winogrande-et",
    pretty_name="Winogrande-et",
    source="EuroEval/winogrande-et",
    task=COMMON_SENSE,
    languages=[ESTONIAN],
    _prompt_prefix="Sulle esitatakse lüngaga (_) tekstülesanded, "
    "igal ülesandel on kaks vastusevarianti (a ja b).",
    _prompt_template="Tekstülesanne: {text}\nVastus: {label}",
    _instruction_prompt="Tekstülesanne: {text}\n\n"
    "Sinu ülesanne on valida lünka sobiv vastusevariant. "
    "Vasta ainult {labels_str}. Muud vastused ei ole lubatud.",
    _labels=["a", "b"],
)

VALEU_ET_CONFIG = DatasetConfig(
    name="valeu-et",
    pretty_name="VaLEU-et",
    source="EuroEval/european-values-et",
    task=EUROPEAN_VALUES,
    languages=[ESTONIAN],
    splits=["test"],
    bootstrap_samples=False,
    _instruction_prompt="{text}",
)

### Unofficial datasets ###

SCALA_ET_CONFIG = DatasetConfig(
    name="scala-et",
    pretty_name="ScaLA-et",
    source="EuroEval/scala-et",
    task=LA,
    languages=[ESTONIAN],
    unofficial=True,
)

EXAM_ET_CONFIG = DatasetConfig(
    name="exam-et",
    pretty_name="Exam-et",
    source="EuroEval/exam-et",
    task=KNOW,
    languages=[ESTONIAN],
    _labels=["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o"],
    unofficial=True,
)

MMLU_ET_CONFIG = DatasetConfig(
    name="mmlu-et",
    pretty_name="MMLU-et",
    source="EuroEval/mmlu-et-mini",
    task=KNOW,
    languages=[ESTONIAN],
    unofficial=True,
)



"""Templates for the Reading Comprehension task."""

import typing as t

from ..data_models import PromptConfig
from ..languages import (
    ALBANIAN,
    BOSNIAN,
    BULGARIAN,
    CATALAN,
    CROATIAN,
    CZECH,
    DANISH,
    DUTCH,
    ENGLISH,
    ESTONIAN,
    FAROESE,
    FINNISH,
    FRENCH,
    GERMAN,
    GREEK,
    HUNGARIAN,
    ICELANDIC,
    ITALIAN,
    LATVIAN,
    LITHUANIAN,
    NORWEGIAN,
    NORWEGIAN_BOKMÅL,
    NORWEGIAN_NYNORSK,
    POLISH,
    PORTUGUESE,
    ROMANIAN,
    SERBIAN,
    SLOVAK,
    SLOVENE,
    SPANISH,
    SWEDISH,
    UKRAINIAN,
)

if t.TYPE_CHECKING:
    from ..languages import Language

RC_TEMPLATES: dict["Language", PromptConfig] = {
    ALBANIAN: PromptConfig(
        default_prompt_prefix="Më poshtë janë tekste me pyetje dhe përgjigje.",
        default_prompt_template="Tekst: {text}\nPyetje: {question}\nPërgjigje me "
        "maksimum 3 fjalë: {label}",
        default_instruction_prompt="Tekst: {text}\n\nPërgjigjuni pyetjes së mëposhtme "
        "rreth tekstit të mësipërm me maksimum 3 fjalë.\n\nPyetje: {question}",
        default_prompt_label_mapping=dict(),
    ),
    BOSNIAN: PromptConfig(
        default_prompt_prefix="Slijede tekstovi s pitanjima i odgovorima.",
        default_prompt_template="Tekst: {text}\nPitanje: {question}\nOdgovor s "
        "najviše 3 riječi: {label}",
        default_instruction_prompt="Tekst: {text}\n\nOdgovorite na sljedeće "
        "pitanje o gornjem tekstu s najviše 3 riječi.\n\nPitanje: {question}",
        default_prompt_label_mapping=dict(),
    ),
    BULGARIAN: PromptConfig(
        default_prompt_prefix="Следват текстове със съответни въпроси и отговори.",
        default_prompt_template="Текст: {text}\nВъпрос: {question}\nОтговор с максимум "
        "3 думи: {label}",
        default_instruction_prompt="Текст: {text}\n\nОтговорете на следния въпрос "
        "относно текста по-горе с максимум 3 думи.\n\nВъпрос: {question}",
        default_prompt_label_mapping=dict(),
    ),
    CATALAN: PromptConfig(
        default_prompt_prefix="Els textos següents contenen preguntes i respostes.",
        default_prompt_template="Text: {text}\nPregunta: {question}\nResposta amb un "
        "màxim de 3 paraules: {label}",
        default_instruction_prompt="Text: {text}\n\nRespon a la següent pregunta "
        "sobre el text anterior amb un màxim de 3 paraules.\n\nPregunta: {question}",
        default_prompt_label_mapping=dict(),
    ),
    CROATIAN: PromptConfig(
        default_prompt_prefix="Sljedeći tekstovi sadrže pitanja i odgovore.",
        default_prompt_template="Tekst: {text}\nPitanje: {question}\nOdgovor s "
        "najviše 3 riječi: {label}",
        default_instruction_prompt="Tekst: {text}\n\nOdgovorite na sljedeće pitanje "
        "o gornjem tekstu s najviše 3 riječi.\n\nPitanje: {question}",
        default_prompt_label_mapping=dict(),
    ),
    CZECH: PromptConfig(
        default_prompt_prefix="Následující texty obsahují otázky a odpovědi.",
        default_prompt_template="Text: {text}\nOtázka: {question}\nOdpověď maximálně "
        "3 slovy: {label}",
        default_instruction_prompt="Text: {text}\n\nOdpovězte na následující otázku k "
        "výše uvedenému textu maximálně 3 slovy.\n\nOtázka: {question}",
        default_prompt_label_mapping=dict(),
    ),
    DANISH: PromptConfig(
        default_prompt_prefix="Følgende er tekster med tilhørende spørgsmål og svar.",
        default_prompt_template="Tekst: {text}\nSpørgsmål: {question}\nSvar med maks. "
        "3 ord: {label}",
        default_instruction_prompt="Tekst: {text}\n\nBesvar følgende spørgsmål om "
        "teksten ovenfor med maks. 3 ord.\n\nSpørgsmål: {question}",
        default_prompt_label_mapping=dict(),
    ),
    GERMAN: PromptConfig(
        default_prompt_prefix="Im Folgenden finden Sie Texte mit den dazugehörigen "
        "Fragen und Antworten.",
        default_prompt_template="Text: {text}\nFragen: {question}\nFragen Antwort in "
        "maximal 3 Wörtern: {label}",
        default_instruction_prompt="Text: {text}\n\nBeantworten Sie die folgende Frage "
        "zum obigen Text in höchstens 3 Wörtern.\n\nFrage: {question}",
        default_prompt_label_mapping=dict(),
    ),
    GREEK: PromptConfig(
        default_prompt_prefix="Ακολουθούν κείμενα με τις αντίστοιχες ερωτήσεις και "
        "απαντήσεις.",
        default_prompt_template="Κείμενο: {text}\nΕρώτηση: {question}\nΑπάντηση σε το "
        "πολύ 3 λέξεις: {label}",
        default_instruction_prompt="Κείμενο: {text}\n\nΑπαντήστε την παρακάτω ερώτηση "
        "σχετικά με το παραπάνω κείμενο σε το πολύ 3 λέξεις.\n\nΕρώτηση: {question}",
        default_prompt_label_mapping=dict(),
    ),
    HUNGARIAN: PromptConfig(
        default_prompt_prefix="Az alábbiakban szövegek szerepelnek a hozzájuk tartozó "
        "kérdésekkel és válaszokkal.",
        default_prompt_template="Szöveg: {text}\nKérdés: {question}\nVálasz "
        "legfeljebb 3 szóban: {label}",
        default_instruction_prompt="Szöveg: {text}\n\nVálaszoljon az alábbi kérdésre "
        "a fenti szöveg alapján legfeljebb 3 szóban.\n\nKérdés: {question}",
        default_prompt_label_mapping=dict(),
    ),
    ENGLISH: PromptConfig(
        default_prompt_prefix="The following are texts with accompanying questions and "
        "answers.",
        default_prompt_template="Text: {text}\nQuestion: {question}\nAnswer in max "
        "3 words: {label}",
        default_instruction_prompt="Text: {text}\n\nAnswer the following question "
        "about the above text in at most 3 words.\n\nQuestion: {question}",
        default_prompt_label_mapping=dict(),
    ),
    SPANISH: PromptConfig(
        default_prompt_prefix="A continuación se presentan textos con sus preguntas y "
        "respuestas correspondientes.",
        default_prompt_template="Texto: {text}\nPregunta: {question}\nRespuesta en "
        "máximo 3 palabras: {label}",
        default_instruction_prompt="Texto: {text}\n\nResponda la siguiente pregunta "
        "sobre el texto anterior en máximo 3 palabras.\n\nPregunta: {question}",
        default_prompt_label_mapping=dict(),
    ),
    ESTONIAN: PromptConfig(
        default_prompt_prefix="Järgnevad on tekstid koos küsimuste ja vastustega.",
        default_prompt_template="Tekst: {text}\nKüsimus: {question}\nVasta "
        "maksimaalselt 3 sõnaga: {label}",
        default_instruction_prompt="Tekst: {text}\n\nVasta järgmisele küsimusele "
        "ülevaltoodud teksti kohta maksimaalselt 3 sõnaga.\n\nKüsimus: {question}",
        default_prompt_label_mapping=dict(),
    ),
    FINNISH: PromptConfig(
        default_prompt_prefix="Seuraavassa on tekstejä ja niihin liittyviä kysymyksiä "
        "ja vastauksia.",
        default_prompt_template="Teksti: {text}\nKysymys: {question} "
        "\nVastaa enintään 3 sanalla: {label}",
        default_instruction_prompt="Teksti: {text}\n\nVastaa seuraavaan "
        "kysymykseen yllä olevasta tekstistä enintään 3 sanalla.\n\n"
        "Kysymys: {question}",
        default_prompt_label_mapping=dict(),
    ),
    FAROESE: PromptConfig(
        default_prompt_prefix="Hetta eru tekstir saman við spurningum og svar.",
        default_prompt_template="Tekstur: {text}\nSpurningur: {question}\nSvara við í "
        "mesta lagi trimum orðum: {label}",
        default_instruction_prompt="Tekstur: {text}\n\nSvara hesum spurninginum um "
        "tekstin uppiyvir við í mesta lagi trimum orðum.\n\nSpurningur: {question}",
        default_prompt_label_mapping=dict(),
    ),
    FRENCH: PromptConfig(
        default_prompt_prefix="Les textes suivants sont accompagnés de questions et de "
        "réponses.",
        default_prompt_template="Texte: {text}\nQuestion: {question}\nRéponse en 3 "
        "mots maximum: {label}",
        default_instruction_prompt="Texte: {text}\n\nRépondez à la question suivante "
        "sur le texte ci-dessus en 3 mots maximum.\n\nQuestion: {question}",
        default_prompt_label_mapping=dict(),
    ),
    ICELANDIC: PromptConfig(
        default_prompt_prefix="Eftirfarandi eru textar með tilheyrandi spurningum og "
        "svörum.",
        default_prompt_template="Texti: {text}\nSpurning: {question}\nSvaraðu með að "
        "hámarki 3 orðum: {label}",
        default_instruction_prompt="Texti: {text}\n\nSvaraðu eftirfarandi spurningu um "
        "textann að hámarki í 3 orðum.\n\nSpurning: {question}",
        default_prompt_label_mapping=dict(),
    ),
    ITALIAN: PromptConfig(
        default_prompt_prefix="I testi che seguono sono accompagnati da domande e "
        "risposte.",
        default_prompt_template="Testo: {text}\nDomanda: {question}\nRispondere in "
        "massimo 3 parole: {label}",
        default_instruction_prompt="Testo: {text}\n\nRispondi alla seguente domanda "
        "sul in un massimo di 3 parole.\n\nDomanda: {question}",
        default_prompt_label_mapping=dict(),
    ),
    LITHUANIAN: PromptConfig(
        default_prompt_prefix="Toliau pateikti tekstai su atitinkamais klausimais ir "
        "atsakymais.",
        default_prompt_template="Tekstas: {text}\nKlausimas: {question}\nAtsakykite ne "
        "daugiau kaip 3 žodžiais: {label}",
        default_instruction_prompt="Tekstas: {text}\n\nAtsakykite į šį klausimą apie "
        "aukščiau pateiktą tekstą ne daugiau kaip 3 žodžiais.\n\nKlausimas: {question}",
        default_prompt_label_mapping=dict(),
    ),
    LATVIAN: PromptConfig(
        default_prompt_prefix="Turpmāk seko teksti ar atbilstošiem jautājumiem un "
        "atbildēm.",
        default_prompt_template="Teksts: {text}\nJautājums: {question}\nAtbildēt ar "
        "maksimāli 3 vārdiem: {label}",
        default_instruction_prompt="Teksts: {text}\n\nAtbildiet uz šo jautājumu par "
        "iepriekš minēto tekstu ar maksimāli 3 vārdiem.\n\nJautājums: {question}",
        default_prompt_label_mapping=dict(),
    ),
    NORWEGIAN_BOKMÅL: PromptConfig(
        default_prompt_prefix="Her følger tekster med tilhørende spørsmål og svar.",
        default_prompt_template="Tekst: {text}\nSpørsmål: {question}\nSvar på maks 3 "
        "ord: {label}",
        default_instruction_prompt="Tekst: {text}\n\nBesvar følgende spørsmål om "
        "teksten ovenfor med maks 3 ord.\n\nSpørsmål: {question}",
        default_prompt_label_mapping=dict(),
    ),
    DUTCH: PromptConfig(
        default_prompt_prefix="Hieronder volgen teksten met bijbehorende vragen en "
        "antwoorden.",
        default_prompt_template="Tekst: {text}\nVraag: {question}\nAntwoord in max "
        "3 woorden: {label}",
        default_instruction_prompt="Tekst: {text}\n\nBeantwoord de volgende vraag "
        "over de bovenstaande tekst in maximaal 3 woorden.\n\nVraag: {question}",
        default_prompt_label_mapping=dict(),
    ),
    NORWEGIAN_NYNORSK: PromptConfig(
        default_prompt_prefix="Her følger tekster med tilhørende spørsmål og svar.",
        default_prompt_template="Tekst: {text}\nSpørsmål: {question}\nSvar på maks 3 "
        "ord: {label}",
        default_instruction_prompt="Tekst: {text}\n\nBesvar følgende spørsmål om "
        "teksten ovenfor med maks 3 ord.\n\nSpørsmål: {question}",
        default_prompt_label_mapping=dict(),
    ),
    NORWEGIAN: PromptConfig(
        default_prompt_prefix="Her følger tekster med tilhørende spørsmål og svar.",
        default_prompt_template="Tekst: {text}\nSpørsmål: {question}\nSvar på maks 3 "
        "ord: {label}",
        default_instruction_prompt="Tekst: {text}\n\nBesvar følgende spørsmål om "
        "teksten ovenfor med maks 3 ord.\n\nSpørsmål: {question}",
        default_prompt_label_mapping=dict(),
    ),
    ROMANIAN: PromptConfig(
        default_prompt_prefix="Iată texte cu întrebări și răspunsuri însoțite.",
        default_prompt_template="Text: {text}\nÎntrebare: {question}\nRăspuns de "
        "maxim 3 cuvinte: {label}",
        default_instruction_prompt="Text: {text}\n\nRăspunde la următoarea întrebare "
        "referitoare la textul de mai sus folosind maxim 3 cuvinte.\n\nÎntrebare: "
        "{question}",
        default_prompt_label_mapping=dict(),
    ),
    POLISH: PromptConfig(
        default_prompt_prefix="Poniżej znajdują się teksty z towarzyszącymi "
        "pytaniami i odpowiedziami.",
        default_prompt_template="Tekst: {text}\nPytanie: {question}\nOdpowiedź z "
        "użyciem maksymalnie 3 słów: {label}",
        default_instruction_prompt="Tekst: {text}\n\nOdpowiedz na następujące pytanie "
        "dotyczące powyższego tekstu, używając maksymalnie 3 słów.\n\nPytanie: "
        "{question}",
        default_prompt_label_mapping=dict(),
    ),
    PORTUGUESE: PromptConfig(
        default_prompt_prefix="Os textos que se seguem são acompanhados de perguntas "
        "e respostas.",
        default_prompt_template="Texto: {text}\nPergunta: {question}\nResposta com "
        "um máximo de 3 palavras: {label}",
        default_instruction_prompt="Texto: {text}\n\nResponde à seguinte pergunta "
        "sobre o texto acima num máximo de 3 palavras.\n\nPergunta: {question}",
        default_prompt_label_mapping=dict(),
    ),
    SLOVENE: PromptConfig(
        default_prompt_prefix="Spodaj so besedila z ustreznimi vprašanji in odgovori.",
        default_prompt_template="Besedilo: {text}\nVprašanje: {question}\nOdgovor v "
        "največ 3 besedah: {label}",
        default_instruction_prompt="Besedilo: {text}\n\nOdgovorite na naslednje "
        "vprašanje o zgornjem besedilu v največ 3 besedah.\n\nVprašanje: {question}",
        default_prompt_label_mapping=dict(),
    ),
    SLOVAK: PromptConfig(
        default_prompt_prefix="Nasledujú texty s pridruženými otázkami a odpoveďami.",
        default_prompt_template="Text: {text}\nOtázka: {question}\nOdpoveď na "
        "maximálne 3 slová: {label}",
        default_instruction_prompt="Text: {text}\n\nOdpovedzte na nasledujúcu "
        "otázku týkajúcu sa textu uvedeného vyššie maximálne 3 slovami.\n\nOtázka: "
        "{question}",
        default_prompt_label_mapping=dict(),
    ),
    SERBIAN: PromptConfig(
        default_prompt_prefix="Следе текстови са одговарајућим питањима и одговорима.",
        default_prompt_template="Текст: {text}\nПитање: {question}\nОдговор у "
        "максимум 3 речи: {label}",
        default_instruction_prompt="Текст: {text}\n\nОдговорите на следеће питање о "
        "горњем тексту у максимум 3 речи.\n\nПитање: {question}",
        default_prompt_label_mapping=dict(),
    ),
    SWEDISH: PromptConfig(
        default_prompt_prefix="Nedan följer texter med tillhörande frågor och svar.",
        default_prompt_template="Text: {text}\nFråga: {question}\nSvar på max 3 ord: "
        "{label}",
        default_instruction_prompt="Text: {text}\n\nBesvara följande fråga om texten "
        "ovan med högst 3 ord.\n\nFråga: {question}",
        default_prompt_label_mapping=dict(),
    ),
    UKRAINIAN: PromptConfig(
        default_prompt_prefix="Нижче наведені тексти з відповідними питаннями та "
        "відповідями.",
        default_prompt_template="Текст: {text}\nПитання: {question}\nВідповідь "
        "максимум 3 словами: {label}",
        default_instruction_prompt="Текст: {text}\n\nВідповідь на наступне питання "
        "про вищезазначений текст максимум 3 словами.\n\nПитання: {question}",
        default_prompt_label_mapping=dict(),
    ),
}



"""All Greek dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import GREEK
from ..tasks import COMMON_SENSE, KNOW, LA, NER, RC, SENT, SUMM

### Official datasets ###

GREEK_SA_CONFIG = DatasetConfig(
    name="greek-sa",
    pretty_name="Greek Sentiment Analysis",
    source="EuroEval/greek-sa-mini",
    task=SENT,
    languages=[GREEK],
    _labels=["negative", "positive"],
)

SCALA_EL_CONFIG = DatasetConfig(
    name="scala-el",
    pretty_name="ScaLA-el",
    source="EuroEval/scala-el",
    task=LA,
    languages=[GREEK],
)

ELNER_CONFIG = DatasetConfig(
    name="elner",
    pretty_name="ElNER",
    source="EuroEval/elner-mini",
    task=NER,
    languages=[GREEK],
)

MULTI_WIKI_QA_EL_CONFIG = DatasetConfig(
    name="multi-wiki-qa-el",
    pretty_name="MultiWikiQA-el",
    source="EuroEval/multi-wiki-qa-el-mini",
    task=RC,
    languages=[GREEK],
)

GREEK_WIKIPEDIA_CONFIG = DatasetConfig(
    name="greek-wikipedia",
    pretty_name="Greek Wikipedia",
    source="EuroEval/greek-wikipedia-mini",
    task=SUMM,
    languages=[GREEK],
)

GLOBAL_MMLU_EL_CONFIG = DatasetConfig(
    name="global-mmlu-el",
    pretty_name="GlobalMMLU-el",
    source="EuroEval/global-mmlu-el-mini",
    task=KNOW,
    languages=[GREEK],
)

WINOGRANDE_EL_CONFIG = DatasetConfig(
    name="winogrande-el",
    pretty_name="Winogrande-el",
    source="EuroEval/winogrande-el",
    task=COMMON_SENSE,
    languages=[GREEK],
)



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
#     "scikit-learn<1.6.0",
# ]
# ///

"""Create the MMLU-LV (Latvian) dataset from VTI-Data and upload to HF Hub."""

from collections import Counter
from typing import Any

import pandas as pd
import requests
from datasets import Dataset, DatasetDict, Split
from huggingface_hub import HfApi
from sklearn.model_selection import train_test_split

from .constants import (
    CHOICES_MAPPING,
    MAX_NUM_CHARS_IN_INSTRUCTION,
    MAX_NUM_CHARS_IN_OPTION,
    MAX_REPETITIONS,
    MIN_NUM_CHARS_IN_INSTRUCTION,
    MIN_NUM_CHARS_IN_OPTION,
)


def main() -> None:
    """Create the MMLU-LV dataset and upload to HF Hub."""
    # Get all subject files
    subjects = get_mmlu_subjects_from_github()
    assert len(subjects) == 57, f"Expected 57 subjects, got {len(subjects)}"

    # Download and process all subjects
    all_data = []
    for subject_info in subjects:
        subject_data = download_subject_data(subject_info)
        if subject_data:
            all_data.extend(subject_data)

    assert len(all_data) == 13941, f"Expected 13941 questions, got {len(all_data)}"

    # Process the data
    df = process_mmlu_data(all_data)

    if df.empty:
        print("No valid data after processing. Exiting.")
        return

    # Apply filtering from the original script
    df = df[
        (df.instruction.str.len() >= MIN_NUM_CHARS_IN_INSTRUCTION)
        & (df.instruction.str.len() <= MAX_NUM_CHARS_IN_INSTRUCTION)
        & (df.option_a.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_a.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_b.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_b.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_c.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_c.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_d.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_d.str.len() <= MAX_NUM_CHARS_IN_OPTION)
    ]

    # Remove overly repetitive samples
    df = df[
        ~df.instruction.apply(is_repetitive)
        & ~df.option_a.apply(is_repetitive)
        & ~df.option_b.apply(is_repetitive)
        & ~df.option_c.apply(is_repetitive)
        & ~df.option_d.apply(is_repetitive)
    ]
    assert isinstance(df, pd.DataFrame)

    # Create the text column with choices in Latvian
    choices_word = CHOICES_MAPPING["lv"]
    df["text"] = [
        row.instruction.replace("\n", " ").strip() + "\n"
        f"{choices_word}:\n"
        "a. " + row.option_a.replace("\n", " ").strip() + "\n"
        "b. " + row.option_b.replace("\n", " ").strip() + "\n"
        "c. " + row.option_c.replace("\n", " ").strip() + "\n"
        "d. " + row.option_d.replace("\n", " ").strip()
        for _, row in df.iterrows()
    ]

    # Ensure label consistency
    df.label = df.label.str.lower()

    # Only keep the required columns
    df = df[["text", "label", "category"]]

    # Remove duplicates
    df.drop_duplicates(inplace=True)
    df.reset_index(drop=True, inplace=True)

    # Create validation split
    val_size = 256
    traintest_arr, val_arr = train_test_split(
        df,
        test_size=val_size,
        random_state=4242,
        stratify=df.category if df.category.nunique() > 1 else None,
    )
    traintest_df = pd.DataFrame(traintest_arr, columns=df.columns)
    val_df = pd.DataFrame(val_arr, columns=df.columns)

    # Create test split
    test_size = 2048
    train_arr, test_arr = train_test_split(
        traintest_df,
        test_size=test_size,
        random_state=4242,
        stratify=traintest_df.category if traintest_df.category.nunique() > 1 else None,
    )
    train_df = pd.DataFrame(train_arr, columns=df.columns)
    test_df = pd.DataFrame(test_arr, columns=df.columns)

    # Create train split
    train_size = 1024
    train_df = train_df.sample(train_size, random_state=4242)

    # Reset indices
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Create dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/mmlu-lv-mini"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    dataset.push_to_hub(dataset_id, private=True)


def get_mmlu_subjects_from_github() -> list[dict[str, str]]:
    """Get all MMLU subject files from the VTI-Data repository.

    Returns:
        List of dictionaries with subject names and download URLs.
    """
    api_url = "https://api.github.com/repos/LUMII-AILab/VTI-Data/contents/mmlu"

    response = requests.get(api_url)
    response.raise_for_status()
    contents = response.json()

    # Extract JSON files and their download URLs
    json_files = [
        {
            "name": item["name"][:-5],  # Remove .json extension
            "download_url": item["download_url"],
        }
        for item in contents
        if item["type"] == "file"
        and item["name"].endswith(".json")
        and item["name"] != "LICENSE"  # Skip license file
    ]

    # Filter out the regular sociology subject, keeping only sociology_postedited
    json_files = [f for f in json_files if f["name"] != "sociology"]

    return sorted(json_files, key=lambda x: x["name"])


def download_subject_data(subject_info: dict[str, str]) -> list[dict[str, Any]]:
    """Download and parse data for a specific MMLU subject.

    Args:
        subject_info: Dictionary with subject name and download URL

    Returns:
        List of question dictionaries
    """
    response = requests.get(subject_info["download_url"])
    response.raise_for_status()
    data = response.json()

    # Add subject category to each question
    for item in data:
        item["category"] = subject_info["name"]

    return data


def process_mmlu_data(data: list[dict[str, Any]]) -> pd.DataFrame:
    """Process raw MMLU data into the expected format.

    Args:
        data: List of raw question dictionaries

    Returns:
        Processed DataFrame with columns: instruction, option_a,
            option_b, option_c, option_d, label, category
    """
    processed_data = []

    for item in data:
        # Handle different possible data structures
        question = item.get("question", item.get("instruction", ""))
        choices = item.get("choices", item.get("options", []))
        answer = item.get("answer", item.get("correct_answer", ""))
        category = item.get("category", "unknown")

        assert isinstance(choices, list), (
            f"Choices should be a list, got {type(choices)}"
        )

        # Ensure we have exactly 4 choices
        if len(choices) != 4:
            continue

        # Convert answer to lowercase letter format if it's numeric
        if isinstance(answer, int):
            answer = ["a", "b", "c", "d"][answer]
        elif isinstance(answer, str) and answer.isdigit():
            answer = ["a", "b", "c", "d"][int(answer)]
        elif isinstance(answer, str):
            answer = answer.lower().strip()

        processed_item = {
            "instruction": question,
            "option_a": choices[0],
            "option_b": choices[1],
            "option_c": choices[2],
            "option_d": choices[3],
            "label": answer,
            "category": category,
        }

        processed_data.append(processed_item)

    return pd.DataFrame(processed_data)


def is_repetitive(text: str) -> bool:
    """Return True if the text is repetitive.

    Args:
        text (str): The text to check.

    Returns:
        bool: True if the text is repetitive, False otherwise.
    """
    if not isinstance(text, str):
        return False
    max_repetitions = max(Counter(text.split()).values()) if text.split() else 0
    return max_repetitions > MAX_REPETITIONS


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = []
# ///

"""Scripts related to updating of version."""

import datetime as dt
import re
import subprocess
from pathlib import Path
from typing import Tuple


def bump_major() -> None:
    """Add one to the major version."""
    major, _, _ = get_current_version()
    set_new_version(major + 1, 0, 0)


def bump_minor() -> None:
    """Add one to the minor version."""
    major, minor, _ = get_current_version()
    set_new_version(major, minor + 1, 0)


def bump_patch() -> None:
    """Add one to the patch version."""
    major, minor, patch = get_current_version()
    set_new_version(major, minor, patch + 1)


def set_new_version(major: int, minor: int, patch: int) -> None:
    """Sets a new version.

    Args:
        major:
            The major version. This only changes when the code stops being backwards
            compatible.
        minor:
            The minor version. This changes when a backwards compatible change
            happened.
        patch:
            The patch version. This changes when the only new changes are bug fixes.
    """
    version = f"{major}.{minor}.{patch}"

    # Get current changelog and ensure that it has an [Unreleased] entry
    changelog_path = Path("CHANGELOG.md")
    changelog = changelog_path.read_text()
    if "[Unreleased]" not in changelog:
        raise RuntimeError("No [Unreleased] entry in CHANGELOG.md.")

    # Add version to CHANGELOG
    today = dt.date.today().strftime("%Y-%m-%d")
    new_changelog = re.sub(
        r"\[Unreleased\].*", f"[Unreleased]\n\n## [v{version}] - {today}", changelog
    )
    changelog_path.write_text(new_changelog)

    # Update the version in the `pyproject.toml` file
    pyproject_path = Path("pyproject.toml")
    pyproject = pyproject_path.read_text()
    pyproject = re.sub(
        r'version = "[^"]+"', f'version = "{version}"', pyproject, count=1
    )
    pyproject_path.write_text(pyproject)

    # Install newest project
    subprocess.run(["make", "install"])

    # Add to version control
    subprocess.run(["git", "add", ".pre-commit-config.yaml"])
    subprocess.run(["git", "add", "CHANGELOG.md"])
    subprocess.run(["git", "add", "pyproject.toml"])
    subprocess.run(["git", "add", "uv.lock"])
    subprocess.run(["git", "commit", "-m", f"feat: v{version}"])
    subprocess.run(["git", "tag", f"v{version}"])
    subprocess.run(["git", "push"])
    subprocess.run(["git", "push", "--tags"])


def get_current_version() -> Tuple[int, int, int]:
    """Fetch the current version of the package.

    Returns:
        The current version, separated into major, minor and patch versions.
    """
    # Get all the version candidates from pyproject.toml
    version_candidates = re.search(
        r'(?<=version = ")[^"]+(?=")', Path("pyproject.toml").read_text()
    )

    # If no version candidates were found, raise an error
    if version_candidates is None:
        raise RuntimeError("No version found in pyproject.toml.")

    # Otherwise, extract the version, split it into major, minor and patch parts and
    # return these
    else:
        version_str = version_candidates.group(0).replace(".dev", "")
        major, minor, patch = map(int, version_str.split("."))
        return major, minor, patch


if __name__ == "__main__":
    from argparse import ArgumentParser

    parser = ArgumentParser()
    parser.add_argument(
        "--major",
        const=True,
        nargs="?",
        default=False,
        help="Bump the major version by one.",
    )
    parser.add_argument(
        "--minor",
        const=True,
        nargs="?",
        default=False,
        help="Bump the minor version by one.",
    )
    parser.add_argument(
        "--patch",
        const=True,
        nargs="?",
        default=False,
        help="Bump the patch version by one.",
    )
    args = parser.parse_args()

    if args.major + args.minor + args.patch != 1:
        raise RuntimeError(
            "Exactly one of --major, --minor and --patch must be selected."
        )
    elif args.major:
        bump_major()
    elif args.minor:
        bump_minor()
    elif args.patch:
        bump_patch()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Lithuanian Emotions dataset and upload it to the HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi
from sklearn.model_selection import train_test_split
from sklearn.utils import resample

from .constants import MAX_NUM_CHARS_IN_DOCUMENT, MIN_NUM_CHARS_IN_DOCUMENT  # noqa


def main() -> None:
    """Create the Lithuanian Emotions dataset and upload it to the HF Hub."""
    # Define the repository ID
    repo_id = "SkyWater21/lt_emotions"

    # Download the dataset
    dataset = load_dataset(path=repo_id, token=True)
    assert isinstance(dataset, DatasetDict)

    # Convert the dataset splits to dataframes
    train_df = dataset["comb_train"].to_pandas()
    val_df = dataset["comb_validation"].to_pandas()
    test_df1 = dataset["lt_go_emotions_test"].to_pandas()
    test_df2 = dataset["lt_twitter_emotions_test"].to_pandas()
    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df1, pd.DataFrame)
    assert isinstance(test_df2, pd.DataFrame)

    # Combine the two test splits
    test_df = pd.concat([test_df1, test_df2], ignore_index=True)

    def process_split(df: pd.DataFrame) -> pd.DataFrame:
        """Process a split by filtering single labels and mapping them.

        Args:
            df: The dataframe to process.

        Returns:
            The processed dataframe.
        """
        # Filter to only keep samples with single labels (ignore multi-label samples)
        df = df.loc[df["labels"].apply(lambda x: len(x) == 1)].copy()

        # Extract the single integer from the labels list
        df["label_int"] = df["labels"].apply(lambda x: x[0])

        # Map the labels to the EuroEval sentiment labels
        label_mapping = {
            0: "negative",  # anger
            1: "negative",  # disgust
            2: "negative",  # fear
            3: "positive",  # joy
            4: "negative",  # sadness
            5: "positive",  # surprise
            6: "neutral",  # neutral
        }
        df["label"] = df["label_int"].map(label_mapping)

        # Remove any unmapped labels
        df = df.dropna(subset=["label"])

        # Keep only the required columns: lt_text -> text, label
        df = df[["lt_text", "label"]].rename(columns={"lt_text": "text"})

        # Filter by text length
        df["text_len"] = df.text.str.len()
        df = df.query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT").query(
            "text_len <= @MAX_NUM_CHARS_IN_DOCUMENT"
        )

        # Drop the temporary text_len column
        df = df.drop(columns=["text_len"])

        # Create a uniform label distribution
        df = _create_uniform_label_distribution(df=df)
        return df

    # Process each split
    train_df = process_split(df=train_df)
    val_df = process_split(df=val_df)
    test_df = process_split(df=test_df)

    train_size = 1024
    val_size = 256
    test_size = 2048

    # Assuming train_df, val_df, and test_df are your original DataFrames
    train_df, _ = train_test_split(
        train_df, train_size=train_size, stratify=train_df["label"], random_state=4242
    )
    val_df, _ = train_test_split(
        val_df, train_size=val_size, stratify=val_df["label"], random_state=4242
    )
    test_df, _ = train_test_split(
        test_df, train_size=test_size, stratify=test_df["label"], random_state=4242
    )

    # Reset indices
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Create DatasetDict
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/lithuanian-emotions-mini"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    dataset.push_to_hub(dataset_id, private=True)


def _create_uniform_label_distribution(
    df: pd.DataFrame, random_state: int = 4242
) -> pd.DataFrame:
    """Create a sampled dataset with a uniform label distribution.

    Args:
        df: The input dataframe with a 'label' column.
        random_state: The random state for reproducibility.

    Returns:
        A dataframe with a uniform label distribution.
    """
    # Separate each class
    classes = df["label"].unique()
    class_dfs = [df[df["label"] == label] for label in classes]

    # Find the size of the smallest class
    min_size = min(len(class_df) for class_df in class_dfs)

    # Resample each class to the size of the smallest class
    resampled_dfs = [
        resample(class_df, replace=False, n_samples=min_size, random_state=random_state)
        for class_df in class_dfs
    ]

    # Combine the resampled dataframes
    balanced_df = pd.concat(resampled_dfs, ignore_index=True)

    return balanced_df


if __name__ == "__main__":
    main()



"""Benchmarking model inference speed."""

import collections.abc as c
import logging
import typing as t

import pyinfer
from transformers.models.auto.tokenization_auto import AutoTokenizer

from .benchmark_modules import HuggingFaceEncoderModel, LiteLLMModel, VLLMModel
from .exceptions import InvalidBenchmark
from .logging_utils import get_pbar, log
from .utils import clear_memory

if t.TYPE_CHECKING:
    from .benchmark_modules import BenchmarkModule
    from .data_models import BenchmarkConfig


def benchmark_speed(
    model: "BenchmarkModule", benchmark_config: "BenchmarkConfig"
) -> c.Sequence[dict[str, float]]:
    """Benchmark model inference speed.

    Args:
        model:
            Model to use.
        benchmark_config:
            Configuration for the benchmark.

    Returns:
        Dictionary of scores.
    """
    scores: list[dict[str, float]] = list()
    for idx in get_pbar(
        iterable=range(benchmark_config.num_iterations),
        desc="Benchmarking",
        disable=not benchmark_config.progress_bar,
    ):
        itr_scores = benchmark_speed_single_iteration(model=model, itr_idx=idx)
        clear_memory()
        scores.append(itr_scores)
        log(f"Scores for iteration {idx}: {itr_scores}", level=logging.DEBUG)
    return scores


def benchmark_speed_single_iteration(
    model: "BenchmarkModule", itr_idx: int
) -> dict[str, float]:
    """Run a single iteration of the speed benchmark.

    Args:
        model:
            The model to use in the benchmark.
        itr_idx:
            The index of the iteration.

    Returns:
        A dictionary containing the scores for the current iteration.
    """
    gpt2_tokeniser = AutoTokenizer.from_pretrained("gpt2", trust_remote_code=True)

    base_doc = "Document which contains roughly 10 tokens. "
    multiplier = 10 * (1 + itr_idx)
    doc = base_doc * multiplier
    short_multiplier = 1.25 * (1 + itr_idx)
    short_doc = base_doc * round(short_multiplier)

    def generate_messages_predict(doc: str) -> None:
        model.generate(inputs=dict(messages=[[dict(role="user", content=doc)]]))

    def generate_prompt_predict(doc: str) -> None:
        model.generate(inputs=dict(text=[doc]))

    def encoder_predict(doc: str) -> None:
        tokeniser = model.get_tokeniser()
        pytorch_model = model.get_pytorch_module()
        inputs = {
            key: tensor.to(pytorch_model.device)
            for key, tensor in tokeniser(
                text=[doc], truncation=True, return_tensors="pt"
            ).items()
        }
        pytorch_model(**inputs)

    if isinstance(model, VLLMModel):
        predict = generate_prompt_predict
    elif isinstance(model, LiteLLMModel):
        predict = generate_messages_predict
    elif isinstance(model, HuggingFaceEncoderModel):
        predict = encoder_predict
    else:
        raise ValueError(f"Model type {model} not supported for speed benchmark")

    try:
        # Do a warmup run, as the first run is always slower
        pyinfer.InferenceReport(model=predict, inputs=base_doc, n_seconds=1).run(
            print_report=False
        )

        speed_scores = pyinfer.InferenceReport(
            model=predict, inputs=doc, n_seconds=3
        ).run(print_report=False)
        num_gpt2_tokens = len(gpt2_tokeniser([doc], truncation=True)["input_ids"][0])
        gpt2_tokens_per_second = speed_scores["Infer(p/sec)"] * num_gpt2_tokens

        speed_scores_short = pyinfer.InferenceReport(
            model=predict, inputs=short_doc, n_seconds=3
        ).run(print_report=False)
        num_gpt2_tokens_short = len(
            gpt2_tokeniser([short_doc], truncation=True)["input_ids"][0]
        )
        gpt2_tokens_per_second_short = (
            speed_scores_short["Infer(p/sec)"] * num_gpt2_tokens_short
        )

    except (RuntimeError, ValueError, IndexError) as e:
        raise InvalidBenchmark(f"Speed benchmark failed with error: {e!r}") from e

    return dict(
        test_speed=gpt2_tokens_per_second, test_speed_short=gpt2_tokens_per_second_short
    )



"""Constants used throughout the project."""

import re
from typing import TypeVar

from .enums import TaskGroup

# Type variable used for generic typing
T = TypeVar("T", bound=object)

# This is used as input to generative models; it cannot be a special token
DUMMY_FILL_VALUE = 100

# This is the maximum allowed context length for models for the purpose of this
# benchmark. We will still report the models' true maximum context length in the
# metadata, but we won't use it for evaluation, as vLLM needs to allocate memory for
# all tokens in the context.
MAX_CONTEXT_LENGTH = 8_192

# We need to raise the amount of tokens generated for reasoning models, to give them
# time to think
REASONING_MAX_TOKENS = 8_192

# The Hugging Face Hub pipeline tags used to classify models as generative
GENERATIVE_PIPELINE_TAGS = [
    "text-generation",
    "text2text-generation",
    "image-text-to-text",
    "audio-text-to-text",
    "video-text-to-text",
]

# Used to disallow non-generative models to be evaluated on these task groups
GENERATIVE_DATASET_TASK_GROUPS = [TaskGroup.TEXT_TO_TEXT]

# Local models are required to have one of these files in their directory
LOCAL_MODELS_REQUIRED_FILES = ["config.json", "adapter_config.json"]

# The number of top log probabilities to return for generative models. For several APIs
# this is the maximum number of log probabilities that can be returned
MAX_VLLM_LOGPROBS = 20
MAX_LITELLM_LOGPROBS = 8

# We make sure to remove these metric attributes after each iteration, to avoid memory
# leaks
METRIC_ATTRIBUTES_TAKING_UP_MEMORY = ["cached_bertscorer"]

# Hugging Face Hub tags used to classify models as merge models
MERGE_TAGS = ["merge", "mergekit"]

# The minimum required CUDA compute capability for using bfloat16 in vLLM
VLLM_BF16_MIN_CUDA_COMPUTE_CAPABILITY = 8.0

# The candidates for end-of-sequence, beginning-of-sequence and padding tokens
EOS_TOKENS = ["</s>", "<|end_of_text|>", "<|endoftext|>", "[SEP]", "<|return|>"]
BOS_TOKENS = ["<s>", "<|begin_of_text|>", "<|startoftext|>", "[CLS]"]
PAD_TOKENS = [
    "<pad>",
    "<PAD>",
    "[pad]",
    "[PAD]",
    "<|endoftext|>",
    "<｜end▁of▁sentence｜>",
    "<|im_end|>",
]

# Used to detect whether a model is a reasoning model
REASONING_TOKENS: list[tuple[str | re.Pattern, str | re.Pattern]] = [
    ("<think>", "</think>"),
    ("<reason>", "</reason>"),
    ("<reasoning>", "</reasoning>"),
    (
        re.compile(pattern=r"<\|channel\|>(analysis|commentary)<\|message\|>"),
        "<|channel|>final<|message|>",
    ),
]

# These tokens are sometimes used by models to indicate the end of a generated
# response, but they do not use them as a proper EOS token, so we have to deal with them
# manually. We only use them as stop tokens if they actually appear in the model's
# output
CUSTOM_STOP_TOKENS = ["<sep>"]

# For classification tasks we force LiteLLM models to output a JSON dictionary with a
# single key and the values being restricted to the allowed labels. This is the key we
# use
LITELLM_CLASSIFICATION_OUTPUT_KEY = "label"

# These characters are stripped from JSON output when trying to identify the label
JSON_STRIP_CHARACTERS = ' {}\n\r":'

# The number of tokens we generate when evaluating generative models on classification
# tasks. We also use this to determine whether we should store logprobs in the model
# outputs (and cache).
NUM_GENERATION_TOKENS_FOR_CLASSIFICATION = 10

# We only allow loading local datasets in these file formats
SUPPORTED_FILE_FORMATS_FOR_LOCAL_DATASETS = ["csv"]

# These are default generation parameters, and can be overridden if a generative model
# has a `generation_config.json` file in its repository
GENERATION_KWARGS = {
    "temperature": 0.0,
    "top_p": 1.0,
    "top_k": 0,
    "repetition_penalty": 1.0,
}

# This is a mirror of `AttentionBackendEnum` in vLLM, but since we don't have access to
# this when running on CPU/MacOS (as we can only run an old vLLM version), we have to
# define it here
ATTENTION_BACKENDS: list[str] = [
    "FLASH_ATTN",
    "FLASH_ATTN_DIFFKV",
    "TRITON_ATTN",
    "ROCM_ATTN",
    "ROCM_AITER_MLA",
    "ROCM_AITER_TRITON_MLA",
    "ROCM_AITER_FA",
    "ROCM_AITER_MLA_SPARSE",
    "TORCH_SDPA",
    "FLASHINFER",
    "FLASHINFER_MLA",
    "TRITON_MLA",
    "CUTLASS_MLA",
    "FLASHMLA",
    "FLASHMLA_SPARSE",
    "FLASH_ATTN_MLA",
    "IPEX",
    "NO_ATTENTION",
    "FLEX_ATTENTION",
    "TREE_ATTN",
    "ROCM_AITER_UNIFIED_ATTN",
    "CPU_ATTN",
    "CUSTOM",
]
